package com.glxp.sale.admin.controller.auth;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.*;
import com.glxp.sale.admin.entity.info.CompanyEntity;
import com.glxp.sale.admin.entity.param.SystemPDFTemplateEntity;
import com.glxp.sale.admin.entity.param.SystemPDFTemplateRelevanceEntity;
import com.glxp.sale.admin.req.auth.CustomerInfoFilterRequest;
import com.glxp.sale.admin.req.info.DeleteRequest;
import com.glxp.sale.admin.req.param.SystemPDFTemplateRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.auth.AuthRoleAdminService;
import com.glxp.sale.admin.service.auth.CustomerInfoService;
import com.glxp.sale.admin.service.auth.UserRegisterService;
import com.glxp.sale.admin.service.auth.impl.CustomerContactService;
import com.glxp.sale.admin.service.info.CompanyService;
import com.glxp.sale.admin.service.param.SystemPDFTemplateRelevanceService;
import com.glxp.sale.admin.service.param.SystemPDFTemplateService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
@Slf4j
public class CustomerController {

    @Resource
    private CompanyService companyService;
    @Resource
    private CustomerInfoService customerInfoService;
    @Resource
    private CustomerContactService customerContactService;
    @Resource
    private UserRegisterService userRegisterService;
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private AuthRoleAdminService authRoleAdminService;
    @Resource
    private SystemPDFTemplateService systemPDFTemplateService;
    @Resource
    private SystemPDFTemplateRelevanceService systemPDFTemplateRelevanceService;


    @GetMapping("salewarehouse/customer/list")
    public BaseResponse filterList(CustomerInfoFilterRequest customerFilterRequest) {

        List<CustomerDetailEntity> customerDetailEntities = customerInfoService.filterDetailCustomer(customerFilterRequest);
        PageInfo<CustomerDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(customerDetailEntities);
        PageSimpleResponse<CustomerDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(customerDetailEntities);

        for (CustomerDetailEntity customerDetailEntity : customerDetailEntities) {
            CompanyEntity companyEntity = companyService.findCompany(customerDetailEntity.getCustomerId());
            if (companyEntity != null) {
                customerDetailEntity.setFilePath(companyEntity.getFilePath());
            }
        }

        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("salewarehouse/customer/detail")
    public BaseResponse customerDetail(CustomerInfoFilterRequest customerFilterRequest) {

        CustomerDetailEntity customerDetailEntity = customerInfoService.selectDetail(customerFilterRequest.getCustomerId());
        if (customerDetailEntity != null) {
            return ResultVOUtils.success(customerDetailEntity);
        }
        return ResultVOUtils.error(500, "用户信息为空");
    }


    @PostMapping("salewarehouse/customer/update")
    public BaseResponse updateCustomer(@RequestBody @Valid CustomerDetailEntity customerDetailEntity,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        CustomerInfoEntity customerInfoEntity = new CustomerInfoEntity();
        BeanUtils.copyProperties(customerDetailEntity, customerInfoEntity);
        boolean b = customerInfoService.updateCustomerInfo(customerInfoEntity);
        if (customerDetailEntity.getUserFlag() != null) {
            UpdateUserflagBean userflagBean = new UpdateUserflagBean();
            userflagBean.setCustomerId(customerDetailEntity.getCustomerId());
            userflagBean.setUserFlag(customerDetailEntity.getUserFlag());
        }

        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

//        if (StringUtils.isNotBlank(customerDetailEntity.getPdfTemplateId())) {
//            SystemPDFTemplateEntity systemPDFTemplateEntity =
//                    systemPDFTemplateService.selectById(customerDetailEntity.getPdfTemplateId());
//            if (systemPDFTemplateEntity != null) {
//                systemPDFTemplateRelevanceService.deleteByCustomerId(customerDetailEntity.getCustomerId());
//                SystemPDFTemplateRelevanceEntity systemPDFTemplateRelevanceEntity = new SystemPDFTemplateRelevanceEntity();
//                systemPDFTemplateRelevanceEntity.setTemplate_id(systemPDFTemplateEntity.getId());
//                systemPDFTemplateRelevanceEntity.setCustomerId(customerDetailEntity.getCustomerId());
//                systemPDFTemplateRelevanceService.insert(systemPDFTemplateRelevanceEntity);
//            }
//        }

        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("salewarehouse/customer/modify")
    public BaseResponse modifyCustomer(@RequestBody @Valid CustomerDetailEntity customerDetailEntity,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        CustomerInfoEntity customerInfoEntity = new CustomerInfoEntity();
        BeanUtils.copyProperties(customerDetailEntity, customerInfoEntity);
        boolean b = customerInfoService.updateCustomerInfo(customerInfoEntity);

        CustomerContactEntity customerContactEntity = new CustomerContactEntity();
        BeanUtils.copyProperties(customerDetailEntity, customerContactEntity);
        boolean c = customerContactService.updateCustomerContact(customerContactEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("salewarehouse/customer/delete")
    public BaseResponse deleteCustomers(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String id = deleteRequest.getId();
        customerInfoService.deleteById(Long.parseLong(id));
        customerContactService.deleteById(Long.parseLong(id));
        userRegisterService.deleteByCustomerId(Long.parseLong(id));
//        systemPDFTemplateRelevanceService.deleteByCustomerId(Long.parseLong(id));


        return ResultVOUtils.success("删除成功");
    }


}
