package com.glxp.sale.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.sale.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.sale.admin.req.basic.FilterBasicThirdSysDetailRequest;
import com.glxp.sale.admin.req.basic.FilterBasicThirdSysRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.basic.BasicThirdSysResponse;
import com.glxp.sale.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.sale.admin.service.basic.BasicThirdSysService;
import com.glxp.sale.admin.service.basic.BasicUnitMaintainService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class BasicThirdSysController {


    @Resource
    private BasicThirdSysService basicThirdSysService;
    @Resource
    private BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    BasicUnitMaintainService unitMaintainService;
    @Resource
    SystemParamConfigService systemParamConfigService;

    @GetMapping("/udiwms/basic/thirdsys/filter")
    public BaseResponse filter(FilterBasicThirdSysRequest filterBasicThirdSysRequest,
                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicThirdSysEntity> basicThirdSysEntities;
//        if (filterBasicThirdSysRequest.getFilter() != null && filterBasicThirdSysRequest.getFilter()) {
//
//            SystemParamConfigEntity thirdSys = systemParamConfigService.selectByParamKey("erp_updateErpData_thirdSys");
//            if (thirdSys != null) {
//                filterBasicThirdSysRequest.setThirdId(thirdSys.getParamValue());
//                filterBasicThirdSysRequest.setMainSys(null);
//            }
//        }

        basicThirdSysEntities = basicThirdSysService.filterBasicThiSys(filterBasicThirdSysRequest);
//        if (filterBasicThirdSysRequest.getIsBuType() != null && filterBasicThirdSysRequest.getIsBuType()) {
//            BasicThirdSysEntity basicThirdSysEntity = new BasicThirdSysEntity();
//            basicThirdSysEntity.setThirdId("local");
//            basicThirdSysEntity.setThirdName("本地系统");
//            basicThirdSysEntities.add(basicThirdSysEntity);
//
//        }

        PageInfo<BasicThirdSysEntity> pageInfo;
        pageInfo = new PageInfo<>(basicThirdSysEntities);
        PageSimpleResponse<BasicThirdSysEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicThirdSysEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }



    @GetMapping("/udiwms/basic/thirdsys/selectIp")
    public BaseResponse selectIp(FilterBasicThirdSysDetailRequest filterBasicThirdSysDetailRequest,
                                 BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysEntity basicThirdSysEntity = basicThirdSysService.selectByThirdId(filterBasicThirdSysDetailRequest.getThirdSysFk());
        return ResultVOUtils.success(basicThirdSysEntity);
    }

    @PostMapping("/udiwms/basic/thirdsys/update")
    public BaseResponse save(@RequestBody @Valid BasicThirdSysEntity basicThirdSysEntity,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (basicThirdSysEntity != null) {
            BasicThirdSysEntity temp = basicThirdSysService.selectByThirdId(basicThirdSysEntity.getThirdId());
            if (temp.getMainSys() != basicThirdSysEntity.getMainSys()) {
                if (udiRelevanceService.isExit() || unitMaintainService.isExit()) {
                    return ResultVOUtils.error(999, "设置主系统失败");
                }
                if ((!temp.getEnabled()) && basicThirdSysEntity.getMainSys()) {
                    if (!basicThirdSysEntity.getEnabled()) {
                        return ResultVOUtils.error(999, "请先启用系统！");
                    }
                }
                if (!basicThirdSysEntity.getEnabled() && basicThirdSysEntity.getMainSys()) {
                    return ResultVOUtils.error(999, "请先启用系统！");
                }
            }
            if (basicThirdSysEntity.getMainSys()) {
                List<BasicThirdSysEntity> basicThirdSysEntities = basicThirdSysService.filterBasicThiSys(new FilterBasicThirdSysRequest());
                for (BasicThirdSysEntity basicThirdSysEntity1 : basicThirdSysEntities) {
                    basicThirdSysEntity1.setMainSys(false);
                    basicThirdSysService.updateBasicThiSys(basicThirdSysEntity1);
                }
            }
            basicThirdSysService.updateBasicThiSys(basicThirdSysEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("更新成功");
    }


    @GetMapping("/udiwms/basic/thirdsys/filterDetail")
    public BaseResponse filterDetail(FilterBasicThirdSysDetailRequest filterBasicThirdSysDetailRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicThirdSysDetailEntity> basicThirdSysEntities;
        basicThirdSysEntities = basicThirdSysDetailService.filterBasicThirdSysDetail(filterBasicThirdSysDetailRequest);
        PageInfo<BasicThirdSysDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(basicThirdSysEntities);
        PageSimpleResponse<BasicThirdSysDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicThirdSysEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/basic/thirdsys/filterDetailByKey")
    public BaseResponse filterDetailByKey(FilterBasicThirdSysDetailRequest filterBasicThirdSysDetailRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.
                selectByKey(filterBasicThirdSysDetailRequest.getKey(), filterBasicThirdSysDetailRequest.getThirdSysFk());
        return ResultVOUtils.success(basicThirdSysDetailEntity);
    }

    @PostMapping("/udiwms/basic/thirdsys/saveDetail")
    public BaseResponse saveDetail(@RequestBody @Valid BasicThirdSysDetailEntity basicThirdSysDetailEntity,
                                   BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (basicThirdSysDetailEntity != null) {
            basicThirdSysDetailService.updateBasicThirdSysDetail(basicThirdSysDetailEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("更新成功");
    }
}
