package com.glxp.sale.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.sale.admin.entity.basic.BasicThirdSysEntity;
import com.glxp.sale.admin.entity.basic.BasicUnitMaintainEntity;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.httpclient.ErpBasicClient;
import com.glxp.sale.admin.req.basic.BasicUnitMaintainFilterRequest;
import com.glxp.sale.admin.req.basic.CombineSingleUnitRequest;
import com.glxp.sale.admin.req.basic.CombineUnitRequest;
import com.glxp.sale.admin.req.basic.FilterBasicThirdSysRequest;
import com.glxp.sale.admin.req.info.DeleteRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.basic.BasicProductThirdSysResponse;
import com.glxp.sale.admin.res.basic.BasicUnitMaintainResponse;
import com.glxp.sale.admin.res.basic.ErpUnitsResponse;
import com.glxp.sale.admin.service.basic.BasicThirdSysDetailService;
import com.glxp.sale.admin.service.basic.BasicThirdSysService;
import com.glxp.sale.admin.service.basic.BasicUnitMaintainService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.thread.BasicUnitImportService;
import com.glxp.sale.admin.util.CustomUtil;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 往来单位信息维护
 */
@RestController
public class BasicUnitMaintainController {

    @Resource
    private BasicUnitMaintainService basicUnitMaintainService;
    @Resource
    BasicThirdSysDetailService basicThirdSysDetailService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BasicThirdSysService basicThirdSysService;
    @Resource
    BasicUnitImportService basicUnitImportService;

    @GetMapping("/udiwms/basic/unit/maintain/filter")
    public BaseResponse filterBasicUnitMaintain(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                                BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicUnitMaintainEntity> basicUnitMaintainEntities;
        basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);
        PageInfo<BasicUnitMaintainEntity> pageInfo = new PageInfo<>(basicUnitMaintainEntities);
        PageSimpleResponse<BasicUnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicUnitMaintainEntities);

        SystemParamConfigEntity systemParamConfigEntityBUMErpId = systemParamConfigService.selectByParamKey("basic_unit_maintain_erp_id");
        SystemParamConfigEntity systemParamConfigEntityBUMThirdId = systemParamConfigService.selectByParamKey("basic_cops_thirdSys");
        Map<String, Object> rMap = new HashMap<>();
        rMap.put("BUMErpIdValue",
                systemParamConfigEntityBUMErpId != null && StringUtils.isNotBlank(systemParamConfigEntityBUMErpId.getParamValue()) ?
                        systemParamConfigEntityBUMErpId.getParamValue() : "0");
        rMap.put("BUMThirdIdValue",
                systemParamConfigEntityBUMThirdId != null && StringUtils.isNotBlank(systemParamConfigEntityBUMThirdId.getParamValue()) ?
                        systemParamConfigEntityBUMThirdId.getParamValue() : "0");
        rMap.put("page", pageSimpleResponse);
        return ResultVOUtils.success(rMap);
    }

    @GetMapping("/udiwms/basic/unit/maintain/filter2")
    public BaseResponse filterBasicUnitMaintain2(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                                 BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicUnitMaintainEntity> basicUnitMaintainEntities;
        basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);
        PageInfo<BasicUnitMaintainEntity> pageInfo;
        pageInfo = new PageInfo<>(basicUnitMaintainEntities);
        PageSimpleResponse<BasicUnitMaintainEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicUnitMaintainEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/basic/unit/maintain/save")
    public BaseResponse save(@RequestBody @Valid BasicUnitMaintainEntity basicUnitMaintainSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
        BeanUtils.copyProperties(basicUnitMaintainSaveRequest, basicUnitMaintainEntity);
        basicUnitMaintainEntity.setErpId(CustomUtil.getId());
        boolean b = basicUnitMaintainService.insertBasicUnitMaintain(basicUnitMaintainEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success();
    }

    @PostMapping("/udiwms/basic/unit/maintain/update")
    public BaseResponse update(@RequestBody BasicUnitMaintainEntity basicUnitMaintainSaveRequest) {
        basicUnitMaintainService.updateById(basicUnitMaintainSaveRequest);
        return ResultVOUtils.success("修改成功");
    }

    @GetMapping("/udiwms/unit/download")
    public BaseResponse downloadBasicUnit(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BasicUnitMaintainEntity> basicUnitMaintainEntities;
        basicUnitMaintainEntities = basicUnitMaintainService.filterList(basicUnitMaintainFilterRequest);

        List<BasicUnitMaintainResponse> basicUnitMaintainResponses = basicUnitMaintainEntities.stream().map(item ->
        {
            BasicUnitMaintainResponse basicUnitMaintainResponse = new BasicUnitMaintainResponse();
            BeanUtils.copyProperties(item, basicUnitMaintainResponse);
            return basicUnitMaintainResponse;
        }).collect(Collectors.toList());
        PageInfo<BasicUnitMaintainResponse> pageInfo;
        pageInfo = new PageInfo<>(basicUnitMaintainResponses);
        PageSimpleResponse<BasicUnitMaintainResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicUnitMaintainResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/basic/unit/maintain/combine")
    public BaseResponse combine(@RequestBody CombineUnitRequest combineRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ErpUnitsResponse> erpUnitsResponses = combineRequest.getCustmandocResponses();
        if (erpUnitsResponses != null && erpUnitsResponses.size() > 0) {
            for (ErpUnitsResponse erpUnitsResponse : erpUnitsResponses) {
                BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
                if (combineRequest.getThirdSys() != null) {
                    if ("thirdId".equals(combineRequest.getThirdSys())) {
                        basicUnitMaintainEntity.setThirdId(erpUnitsResponse.getId());
                        basicUnitMaintainEntity.setThirdName(erpUnitsResponse.getName());
                    } else if ("thirdId1".equals(combineRequest.getThirdSys())) {
                        basicUnitMaintainEntity.setThirdId1(erpUnitsResponse.getId());
                        basicUnitMaintainEntity.setThirdName1(erpUnitsResponse.getName());
                    } else if ("thirdId2".equals(combineRequest.getThirdSys())) {
                        basicUnitMaintainEntity.setThirdId2(erpUnitsResponse.getId());
                        basicUnitMaintainEntity.setThirdName2(erpUnitsResponse.getName());
                    } else if ("thirdId3".equals(combineRequest.getThirdSys())) {
                        basicUnitMaintainEntity.setThirdId3(erpUnitsResponse.getId());
                        basicUnitMaintainEntity.setThirdName3(erpUnitsResponse.getName());
                    } else if ("thirdId4".equals(combineRequest.getThirdSys())) {
                        basicUnitMaintainEntity.setThirdId4(erpUnitsResponse.getId());
                        basicUnitMaintainEntity.setThirdName4(erpUnitsResponse.getName());
                    }
                }
                basicUnitMaintainEntity.setErpId(CustomUtil.getId());
                basicUnitMaintainEntity.setName(erpUnitsResponse.getName());
                basicUnitMaintainEntity.setSpell(erpUnitsResponse.getSpell());
                basicUnitMaintainEntity.setAddr(erpUnitsResponse.getAddr());
                basicUnitMaintainEntity.setCreditNo(erpUnitsResponse.getCreditNo());
                basicUnitMaintainEntity.setContact(erpUnitsResponse.getContact());
                basicUnitMaintainEntity.setMobile(erpUnitsResponse.getMobile());
                basicUnitMaintainEntity.setUpdateTime(new Date());
                basicUnitMaintainService.insertBasicUnitMaintain(basicUnitMaintainEntity);
            }
        }
        return ResultVOUtils.success("选入成功");
    }


    @GetMapping("/udiwms/basic/unit/maintain/combineAll")
    public BaseResponse combineAll(BasicUnitMaintainFilterRequest unitMaintainFilterRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicThirdSysDetailEntity basicThirdSysDetailEntity = basicThirdSysDetailService.selectByKey("corpUrl", unitMaintainFilterRequest.getThirdSys());
        if (basicThirdSysDetailEntity == null || basicThirdSysDetailEntity.getValue() == null)
            return ResultVOUtils.error(500, "往来单位接口地址未定义");
        if (basicThirdSysDetailEntity.getEnabled()) {
            basicUnitImportService.selectAllUnit(basicThirdSysDetailEntity, unitMaintainFilterRequest);

        } else {
            return ResultVOUtils.error(500, "第三方往来单位服务未启用");
        }


        return ResultVOUtils.success("后台正在添加，请稍后刷新查看！");
    }

    @PostMapping("/udiwms/basic/unit/maintain/combineSingle")
    public BaseResponse combineSingle(@RequestBody CombineSingleUnitRequest combineRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        ErpUnitsResponse erpUnitsResponse = combineRequest.getErpUnitsResponse();
        BasicUnitMaintainEntity basicUnitMaintainEntity = new BasicUnitMaintainEntity();
        basicUnitMaintainEntity.setId(Integer.parseInt(combineRequest.getKey()));
        if (combineRequest.getThirdSys() != null) {
            if ("thirdId".equals(combineRequest.getThirdSys())) {
                basicUnitMaintainEntity.setThirdId(erpUnitsResponse.getId());
                basicUnitMaintainEntity.setThirdName(erpUnitsResponse.getName());
            } else if ("thirdId1".equals(combineRequest.getThirdSys())) {
                basicUnitMaintainEntity.setThirdId1(erpUnitsResponse.getId());
                basicUnitMaintainEntity.setThirdName1(erpUnitsResponse.getName());
            } else if ("thirdId2".equals(combineRequest.getThirdSys())) {
                basicUnitMaintainEntity.setThirdId2(erpUnitsResponse.getId());
                basicUnitMaintainEntity.setThirdName2(erpUnitsResponse.getName());
            } else if ("thirdId3".equals(combineRequest.getThirdSys())) {
                basicUnitMaintainEntity.setThirdId3(erpUnitsResponse.getId());
                basicUnitMaintainEntity.setThirdName3(erpUnitsResponse.getName());
            } else if ("thirdId4".equals(combineRequest.getThirdSys())) {
                basicUnitMaintainEntity.setThirdId4(erpUnitsResponse.getId());
                basicUnitMaintainEntity.setThirdName4(erpUnitsResponse.getName());
            }
        }
        basicUnitMaintainEntity.setThirdName(erpUnitsResponse.getName());
//        basicUnitMaintainEntity.setSpell(erpUnitsResponse.getSpell());
//        basicUnitMaintainEntity.setAddr(erpUnitsResponse.getAddr());
//        basicUnitMaintainEntity.setCreditNo(erpUnitsResponse.getCreditNo());
//        basicUnitMaintainEntity.setContact(erpUnitsResponse.getContact());
//        basicUnitMaintainEntity.setMobile(erpUnitsResponse.getMobile());
        basicUnitMaintainService.updateById(basicUnitMaintainEntity);
        return ResultVOUtils.success("关联成功");
    }



    @PostMapping("/udiwms/basic/unit/maintain/delete")
    public BaseResponse deleteBasicUnitMaintain(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        basicUnitMaintainService.deleteById(id);

        return ResultVOUtils.success("删除成功");
    }



    //获取产品信息详情（多平台）
    @GetMapping("/udiwms/unitMaintain/thirdSys/detail")
    public BaseResponse getThirdSysDetail(BasicUnitMaintainFilterRequest basicUnitMaintainFilterRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        BasicUnitMaintainEntity basicUnitMaintainEntity = basicUnitMaintainService.selectById(basicUnitMaintainFilterRequest.getId() + "");

        FilterBasicThirdSysRequest filterBasicThirdSysRequest = new FilterBasicThirdSysRequest();
        filterBasicThirdSysRequest.setEnabled(true);
        List<BasicThirdSysEntity> basicThirdSysEntities = basicThirdSysService.filterBasicThiSys(filterBasicThirdSysRequest);
        List<BasicProductThirdSysResponse> basicProductThirdSysResponses = new ArrayList<>();

        for (BasicThirdSysEntity basicThirdSysEntity : basicThirdSysEntities) {
            if (basicThirdSysEntity.getThirdId().equals("thirdId")) {
                BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                if (basicUnitMaintainEntity != null) {
                    basicProductThirdSysResponse.setThirdId(basicUnitMaintainEntity.getThirdId());
                    basicProductThirdSysResponse.setThirdName(basicUnitMaintainEntity.getThirdName());
                }

                basicProductThirdSysResponses.add(basicProductThirdSysResponse);
            } else if (basicThirdSysEntity.getThirdId().equals("thirdId1")) {
                BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                if (basicUnitMaintainEntity != null) {
                    basicProductThirdSysResponse.setThirdId(basicUnitMaintainEntity.getThirdId1());
                    basicProductThirdSysResponse.setThirdName(basicUnitMaintainEntity.getThirdName1());
                }
                basicProductThirdSysResponses.add(basicProductThirdSysResponse);
            } else if (basicThirdSysEntity.getThirdId().equals("thirdId2")) {
                BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                if (basicUnitMaintainEntity != null) {
                    basicProductThirdSysResponse.setThirdId(basicUnitMaintainEntity.getThirdId2());
                    basicProductThirdSysResponse.setThirdName(basicUnitMaintainEntity.getThirdName2());
                }
                basicProductThirdSysResponses.add(basicProductThirdSysResponse);
            } else if (basicThirdSysEntity.getThirdId().equals("thirdId3")) {
                BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                if (basicUnitMaintainEntity != null) {
                    basicProductThirdSysResponse.setThirdId(basicUnitMaintainEntity.getThirdId3());
                    basicProductThirdSysResponse.setThirdName(basicUnitMaintainEntity.getThirdName3());
                }
                basicProductThirdSysResponses.add(basicProductThirdSysResponse);
            } else if (basicThirdSysEntity.getThirdId().equals("thirdId4")) {
                BasicProductThirdSysResponse basicProductThirdSysResponse = new BasicProductThirdSysResponse();
                basicProductThirdSysResponse.setSysId(basicThirdSysEntity.getThirdId());
                basicProductThirdSysResponse.setSysName(basicThirdSysEntity.getThirdName());
                if (basicUnitMaintainEntity != null) {
                    basicProductThirdSysResponse.setThirdId(basicUnitMaintainEntity.getThirdId4());
                    basicProductThirdSysResponse.setThirdName(basicUnitMaintainEntity.getThirdName4());
                }
                basicProductThirdSysResponses.add(basicProductThirdSysResponse);
            }
        }
        PageInfo<BasicProductThirdSysResponse> pageInfo;
        pageInfo = new PageInfo<>(basicProductThirdSysResponses);
        PageSimpleResponse<BasicProductThirdSysResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicProductThirdSysResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    public BasicThirdSysEntity filterThirdSys(String thirdId, List<BasicThirdSysEntity> basicThirdSysEntities) {
        if (basicThirdSysEntities != null && basicThirdSysEntities.size() > 0) {
            for (BasicThirdSysEntity basicThirdSysEntity : basicThirdSysEntities) {
                if (thirdId.equals(basicThirdSysEntity.getThirdId())) {
                    return basicThirdSysEntity;
                }
            }
        }
        return null;
    }
}
