package com.glxp.sale.admin.controller.basic;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.basic.BussinessLocalTypeEntity;
import com.glxp.sale.admin.req.basic.BussinessLocalTypeFilterRequest;
import com.glxp.sale.admin.req.info.DeleteRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.basic.BussinessLocalTypeService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class BussinessLocalTypeController {


    @Resource
    private BussinessLocalTypeService bussinessTypeService;


    @GetMapping("/udiwms/localBusType/filterJoin")
    public BaseResponse filterJoin(BussinessLocalTypeFilterRequest bussinessTypeFilterRequest,
                                   BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessLocalTypeEntity> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterJoin(
                bussinessTypeFilterRequest);
        PageInfo<BussinessLocalTypeEntity> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessLocalTypeEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwms/localBusType/filter")
    public BaseResponse filterBussinessType(BussinessLocalTypeFilterRequest bussinessTypeFilterRequest,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<BussinessLocalTypeEntity> bussinessTypeEntities;
        bussinessTypeEntities = bussinessTypeService.filterList(bussinessTypeFilterRequest);
        PageInfo<BussinessLocalTypeEntity> pageInfo;
        pageInfo = new PageInfo<>(bussinessTypeEntities);
        PageSimpleResponse<BussinessLocalTypeEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(bussinessTypeEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/udiwms/localBusType/delete")
    public BaseResponse deleteBussinessType(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        bussinessTypeService.deleteById(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/localBusType/update")
    public BaseResponse updateBussinessType(@RequestBody BussinessLocalTypeEntity bussinessTypeEntity,
                                            BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (bussinessTypeEntity != null) {
            bussinessTypeService.updateBusLocalType(bussinessTypeEntity);
        } else {
            ResultVOUtils.error(999, "参数错误");
        }
        return ResultVOUtils.success("更新成功");
    }


}
