package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.info.CompanyCertEntity;
import com.glxp.sale.admin.entity.inout.StockOrderDetailEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.info.CompanyRequest;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.DeleteRequest;
import com.glxp.sale.admin.req.info.FilterCompanyCertRequest;
import com.glxp.sale.admin.req.inout.StockOrderDetailFilterRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.CompanyCertService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class CompanyCertController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private CompanyCertService companyCertService;

    @GetMapping("/sale/info/company/cert/filter")
    public BaseResponse filterCompanyCert(FilterCompanyCertRequest filterCompanyCertRequest,
                                               BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CompanyCertEntity> companyCertEntityList
                = companyCertService.filterCompanyCert(filterCompanyCertRequest);
        PageInfo<CompanyCertEntity> pageInfo;
        pageInfo = new PageInfo<>(companyCertEntityList);
        PageSimpleResponse<CompanyCertEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyCertEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/sale/info/companyCert")
    public BaseResponse getCompanyCertInfo(CompanyRequest companyRequest) {
        CompanyCertEntity companyCertEntity = companyCertService.findCompanyCert(companyRequest.getCustomerId());
        if (companyCertEntity != null) {
            return ResultVOUtils.success(companyCertEntity);
        }
        return ResultVOUtils.error(500, "空");
    }

    @GetMapping("/sale/info/getCompanyCert")
    public BaseResponse getCompanyCert(FilterCompanyCertRequest filterCompanyCertRequest) {
        filterCompanyCertRequest.setCustomerId(getCustomerId());
        List<CompanyCertEntity> companyCertEntities = companyCertService.getCompanyCert(filterCompanyCertRequest);
        PageInfo<CompanyCertEntity> pageInfo = new PageInfo<>(companyCertEntities);
        PageSimpleResponse<CompanyCertEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyCertEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/sale/info/insertCompanyCert")
    public BaseResponse insertCompanyCert(@RequestBody CompanyCertEntity companyCertEntity) {
        companyCertEntity.setCreate_time(new Date());
        companyCertEntity.setUpdate_time(new Date());
        boolean b = companyCertService.insertCompanyCert(companyCertEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/updateCompanyCert")
    public BaseResponse updateCompanyCert(@RequestBody CompanyCertEntity companyCertEntity) {

        boolean b = companyCertService.updateCompanyCert(companyCertEntity);
        return ResultVOUtils.success("修改成功");
    }

    @PostMapping("/sale/info/deleteCompanyCert")
    public BaseResponse deleteCompanyCert(@RequestBody DeleteCompanyFileRequest deleteCompanyFileRequest, BindingResult bindingResult) {
        boolean b = companyCertService.deleteById(deleteCompanyFileRequest.getId());
        String URL="d:/1s/udiwms/register/file/image2/" + deleteCompanyFileRequest.getFilePath();
        File file=new File(URL);
        if(file.exists()&&file.isFile()) {
            file.delete();
        }
        return ResultVOUtils.success("成功");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
