package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.info.CompanyProductRelevanceEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.basic.CompanyProductRelevanceRequest;
import com.glxp.sale.admin.req.basic.UdiInfoExportRequest;
import com.glxp.sale.admin.req.info.CompanyRequest;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.FilterCompanyProductRelevanceRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.basic.CompanyProductRelevanceResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.CompanyProductRelevanceService;
import com.glxp.sale.admin.thread.CompanyProductsRelService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

@RestController
public class CompanyProductRelevanceController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private CompanyProductRelevanceService companyProductRelevanceService;
    @Resource
    private CompanyProductsRelService companyProductsRelService;


    @GetMapping("/sale/info/company/product/relevance/filter")
    public BaseResponse filterCompanyProductRelevance(CompanyProductRelevanceRequest companyProductRelevanceResponse,
                                                      BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<CompanyProductRelevanceResponse> basicInstrumentMaintainRelevanceResponses = null;
        basicInstrumentMaintainRelevanceResponses = companyProductRelevanceService.filterUdiGp(companyProductRelevanceResponse);
        if (basicInstrumentMaintainRelevanceResponses != null && basicInstrumentMaintainRelevanceResponses.size() > 0) {
            for (CompanyProductRelevanceResponse basicInstrumentMaintainRelevanceResponse : basicInstrumentMaintainRelevanceResponses) {
                if (basicInstrumentMaintainRelevanceResponse.getDiType() == 2 && basicInstrumentMaintainRelevanceResponse.getIsUseDy() != 1) {
                    basicInstrumentMaintainRelevanceResponse.setNameCode(basicInstrumentMaintainRelevanceResponse.getSjcpbm());
                }
            }
        }

        PageInfo<CompanyProductRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(basicInstrumentMaintainRelevanceResponses);
        PageSimpleResponse<CompanyProductRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicInstrumentMaintainRelevanceResponses);

        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/sale/info/companyProductRelevance")
    public BaseResponse getCompanyProductRelevanceInfo(CompanyRequest companyRequest) {
        CompanyProductRelevanceEntity companyProductRelevanceEntity = companyProductRelevanceService.findCompanyProductRelevance(companyRequest.getCustomerId());
        if (companyProductRelevanceEntity != null) {
            return ResultVOUtils.success(companyProductRelevanceEntity);
        }
        return ResultVOUtils.error(500, "空");
    }

    @GetMapping("/sale/info/getCompanyProductRelevance")
    public BaseResponse getCompanyProductRelevance(FilterCompanyProductRelevanceRequest filterCompanyProductRelevanceRequest) {
//        filterCompanyProductRelevanceRequest.setCustomerId(getCustomerId());
        List<CompanyProductRelevanceEntity> companyProductRelevanceEntities = companyProductRelevanceService.getCompanyProductRelevance(filterCompanyProductRelevanceRequest);
        PageInfo<CompanyProductRelevanceEntity> pageInfo = new PageInfo<>(companyProductRelevanceEntities);
        PageSimpleResponse<CompanyProductRelevanceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyProductRelevanceEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/sale/info/insertCompanyProductRelevance")
    public BaseResponse insertCompanyProductRelevance(@RequestBody CompanyProductRelevanceEntity companyProductRelevanceEntity) {
        if (companyProductRelevanceService.findCompanyProductRelevanceByProductUuid(companyProductRelevanceEntity) != null) {
            return ResultVOUtils.error(ResultEnum.DATA_REPEAT);
        }
        companyProductRelevanceEntity.setCreate_time(new Date());
        companyProductRelevanceEntity.setUpdate_time(new Date());
        companyProductRelevanceEntity.setAuditStatus("3");
        boolean b = companyProductRelevanceService.insertCompanyProductRelevance(companyProductRelevanceEntity);
        return ResultVOUtils.success("成功");
    }


    @PostMapping("/sale/info/insertFilter")
    public BaseResponse insertFilter(@RequestBody UdiInfoExportRequest udiInfoExportRequest) {

        companyProductsRelService.insertCompanyProducts(udiInfoExportRequest);
        return ResultVOUtils.success("后台正在导入，请稍后刷新查看！");
    }


    @PostMapping("/sale/info/updateCompanyProductRelevance")
    public BaseResponse updateCompanyProductRelevance(@RequestBody CompanyProductRelevanceEntity companyProductRelevanceEntity) {
        companyProductRelevanceEntity.setUpdate_time(new Date());
        boolean b = companyProductRelevanceService.updateCompanyProductRelevance(companyProductRelevanceEntity);
        return ResultVOUtils.success("修改成功");
    }

    @PostMapping("/sale/info/deleteCompanyProductRelevance")
    public BaseResponse deleteCompanyProductRelevance(@RequestBody DeleteCompanyFileRequest deleteCompanyFileRequest, BindingResult bindingResult) {
        boolean b = companyProductRelevanceService.deleteById(deleteCompanyFileRequest.getId());
        return ResultVOUtils.success("成功");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
