package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.info.CompanySalesmanEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.info.CompanyRequest;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.FilterCompanySalesmanRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.CompanySalesmanService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class CompanySalesmanController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private CompanySalesmanService companySalesmanService;

    @GetMapping("/sale/info/company/salesman/filter")
    public BaseResponse filterCompanySalesman(FilterCompanySalesmanRequest filterCompanySalesmanRequest,
                                               BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CompanySalesmanEntity> companySalesmanEntityList
                = companySalesmanService.filterCompanySalesman(filterCompanySalesmanRequest);
        PageInfo<CompanySalesmanEntity> pageInfo;
        pageInfo = new PageInfo<>(companySalesmanEntityList);
        PageSimpleResponse<CompanySalesmanEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companySalesmanEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/sale/info/companySalesman")
    public BaseResponse getCompanySalesmanInfo(CompanyRequest companyRequest) {
        CompanySalesmanEntity companySalesmanEntity = companySalesmanService.findCompanySalesman(companyRequest.getCustomerId());
        if (companySalesmanEntity != null) {
            return ResultVOUtils.success(companySalesmanEntity);
        }
        return ResultVOUtils.error(500, "空");
    }

    @GetMapping("/sale/info/getCompanySalesman")
    public BaseResponse getCompanySalesman(FilterCompanySalesmanRequest filterCompanySalesmanRequest) {
        filterCompanySalesmanRequest.setCustomerId(getCustomerId());
        List<CompanySalesmanEntity> companySalesmanEntities = companySalesmanService.getCompanySalesman(filterCompanySalesmanRequest);
        PageInfo<CompanySalesmanEntity> pageInfo = new PageInfo<>(companySalesmanEntities);
        PageSimpleResponse<CompanySalesmanEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companySalesmanEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/sale/info/insertCompanySalesman")
    public BaseResponse insertCompanySalesman(@RequestBody CompanySalesmanEntity companySalesmanEntity) {
        companySalesmanEntity.setCreate_time(new Date());
        companySalesmanEntity.setUpdate_time(new Date());
        companySalesmanEntity.setAuditStatus("3");
        boolean b = companySalesmanService.insertCompanySalesman(companySalesmanEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/updateCompanySalesman")
    public BaseResponse updateCompanySalesman(@RequestBody CompanySalesmanEntity companySalesmanEntity) {
        companySalesmanEntity.setUpdate_time(new Date());
        if (StringUtils.isNotBlank(companySalesmanEntity.getNewFilePath())) {
            if (StringUtils.isNotBlank(companySalesmanEntity.getFilePath())) {
                String URL="d:/1s/udiwms/register/file/image5/" + companySalesmanEntity.getFilePath();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
            companySalesmanEntity.setFilePath(companySalesmanEntity.getNewFilePath());
        }
        if (StringUtils.isNotBlank(companySalesmanEntity.getNewFilePath2())) {
            if (StringUtils.isNotBlank(companySalesmanEntity.getFilePath2())) {
                String URL="d:/1s/udiwms/register/file/image5/" + companySalesmanEntity.getFilePath2();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
            companySalesmanEntity.setFilePath2(companySalesmanEntity.getNewFilePath2());
        }
        companySalesmanEntity.setAuditStatus("4");
        boolean b = companySalesmanService.updateCompanySalesman(companySalesmanEntity);
        return ResultVOUtils.success("修改成功");
    }

    @PostMapping("/sale/info/deleteCompanySalesman")
    public BaseResponse deleteCompanySalesman(@RequestBody DeleteCompanyFileRequest deleteCompanyFileRequest, BindingResult bindingResult) {
        boolean b = companySalesmanService.deleteById(deleteCompanyFileRequest.getId());
        if (StringUtils.isNotBlank(deleteCompanyFileRequest.getFilePath())) {
            String URL="d:/1s/udiwms/register/file/image5/" + deleteCompanyFileRequest.getFilePath();
            File file=new File(URL);
            if(file.exists()&&file.isFile()) {
                file.delete();
            }
        }
        if (StringUtils.isNotBlank(deleteCompanyFileRequest.getFilePath2())) {
            String URL="d:/1s/udiwms/register/file/image5/" + deleteCompanyFileRequest.getFilePath2();
            File file=new File(URL);
            if(file.exists()&&file.isFile()) {
                file.delete();
            }
        }
        return ResultVOUtils.success("成功");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
