package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.info.CompanySalesmanEntity;
import com.glxp.sale.admin.entity.info.CompanySalesmanUpdateLogEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.info.CompanyRequest;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.FilterCompanySalesmanUpdateLogRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.CompanySalesmanService;
import com.glxp.sale.admin.service.info.CompanySalesmanUpdateLogService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class CompanySalesmanUpdateLogController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private CompanySalesmanUpdateLogService companySalesmanUpdateLogService;
    @Resource
    private CompanySalesmanService companySalesmanService;

    @GetMapping("/sale/info/company/salesman/update/log/filter")
    public BaseResponse filterCompanySalesmanUpdateLog(FilterCompanySalesmanUpdateLogRequest filterCompanySalesmanUpdateLogRequest,
                                               BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<CompanySalesmanUpdateLogEntity> companySalesmanUpdateLogEntityList
                = companySalesmanUpdateLogService.filterCompanySalesmanUpdateLog(filterCompanySalesmanUpdateLogRequest);
        PageInfo<CompanySalesmanUpdateLogEntity> pageInfo;
        pageInfo = new PageInfo<>(companySalesmanUpdateLogEntityList);
        PageSimpleResponse<CompanySalesmanUpdateLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companySalesmanUpdateLogEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/sale/info/companySalesmanUpdateLog")
    public BaseResponse getCompanySalesmanUpdateLogInfo(CompanyRequest companyRequest) {
        CompanySalesmanUpdateLogEntity companySalesmanUpdateLogEntity = companySalesmanUpdateLogService.findCompanySalesmanUpdateLog(companyRequest.getCustomerId());
        if (companySalesmanUpdateLogEntity != null) {
            return ResultVOUtils.success(companySalesmanUpdateLogEntity);
        }
        return ResultVOUtils.error(500, "空");
    }

    @GetMapping("/sale/info/getCompanySalesmanUpdateLog")
    public BaseResponse getCompanySalesmanUpdateLog(FilterCompanySalesmanUpdateLogRequest filterCompanySalesmanUpdateLogRequest) {
        filterCompanySalesmanUpdateLogRequest.setCustomerId(getCustomerId());
        List<CompanySalesmanUpdateLogEntity> companySalesmanUpdateLogEntities = companySalesmanUpdateLogService.getCompanySalesmanUpdateLog(filterCompanySalesmanUpdateLogRequest);
        PageInfo<CompanySalesmanUpdateLogEntity> pageInfo = new PageInfo<>(companySalesmanUpdateLogEntities);
        PageSimpleResponse<CompanySalesmanUpdateLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companySalesmanUpdateLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/sale/info/insertCompanySalesmanUpdateLog")
    public BaseResponse insertCompanySalesmanUpdateLog(@RequestBody CompanySalesmanUpdateLogEntity companySalesmanUpdateLogEntity) {
        CompanySalesmanEntity companySalesmanEntity = new CompanySalesmanEntity();
        companySalesmanEntity.setId(Integer.parseInt(String.valueOf(companySalesmanUpdateLogEntity.getSalesmanId())));
        companySalesmanEntity.setAuditStatus("4");
        companySalesmanEntity.setUpdate_time(new Date());
        companySalesmanService.updateCompanySalesman(companySalesmanEntity);

        companySalesmanUpdateLogEntity.setCreate_time(new Date());
        companySalesmanUpdateLogEntity.setStatus("1");
        companySalesmanUpdateLogEntity.setLogType("2");//变更审核
        boolean b = companySalesmanUpdateLogService.insertCompanySalesmanUpdateLog(companySalesmanUpdateLogEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/updateCompanySalesmanUpdateLog")
    public BaseResponse updateCompanySalesmanUpdateLog(@RequestBody CompanySalesmanUpdateLogEntity companySalesmanUpdateLogEntity) {
        CompanySalesmanEntity companySalesmanEntity = new CompanySalesmanEntity();
        companySalesmanEntity.setId(Integer.parseInt(String.valueOf(companySalesmanUpdateLogEntity.getSalesmanId())));
        String auditStatus = "";
        if (companySalesmanUpdateLogEntity.getStatus().equals("2")) {
            auditStatus = "3";
        } else if (companySalesmanUpdateLogEntity.getStatus().equals("3")) {
            auditStatus = "5";
        }
        companySalesmanEntity.setAuditStatus(auditStatus);
        companySalesmanEntity.setUpdate_time(new Date());
        companySalesmanService.updateCompanySalesman(companySalesmanEntity);
        companySalesmanUpdateLogEntity.setUpdate_time(new Date());
        FilterCompanySalesmanUpdateLogRequest filterCompanySalesmanUpdateLogRequest = new FilterCompanySalesmanUpdateLogRequest();
        filterCompanySalesmanUpdateLogRequest.setPage(1);
        filterCompanySalesmanUpdateLogRequest.setLimit(1);
        filterCompanySalesmanUpdateLogRequest.setCustomerId(String.valueOf(companySalesmanUpdateLogEntity.getCustomerId()));
        filterCompanySalesmanUpdateLogRequest.setSalesmanId(String.valueOf(companySalesmanUpdateLogEntity.getSalesmanId()));
        List<CompanySalesmanUpdateLogEntity> companySalesmanUpdateLogEntities =
                companySalesmanUpdateLogService.filterCompanySalesmanUpdateLog(filterCompanySalesmanUpdateLogRequest);
        if (companySalesmanUpdateLogEntities.size() > 0) {
            CompanySalesmanUpdateLogEntity tCompanySalesmanUpdateLogEntity = companySalesmanUpdateLogEntities.get(0);
            companySalesmanUpdateLogEntity.setId(tCompanySalesmanUpdateLogEntity.getId());
        }
        boolean b = companySalesmanUpdateLogService.updateCompanySalesmanUpdateLog(companySalesmanUpdateLogEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/deleteCompanySalesmanUpdateLog")
    public BaseResponse deleteCompanySalesmanUpdateLog(@RequestBody DeleteCompanyFileRequest deleteCompanyFileRequest, BindingResult bindingResult) {
        boolean b = companySalesmanUpdateLogService.deleteById(deleteCompanyFileRequest.getId());
        return ResultVOUtils.success("成功");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
