package com.glxp.sale.admin.controller.info;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.auth.CustomerContactEntity;
import com.glxp.sale.admin.entity.info.CompanyEntity;
import com.glxp.sale.admin.entity.info.CompanySalesmanEntity;
import com.glxp.sale.admin.entity.info.CompanyUpdateLogEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.info.CompanyRequest;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.FilterCompanyUpdateLogRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.CompanySalesmanService;
import com.glxp.sale.admin.service.info.CompanyService;
import com.glxp.sale.admin.service.info.CompanyUpdateLogService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class CompanyUpdateLogController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private CompanyUpdateLogService companyUpdateLogService;
    @Resource
    private CompanySalesmanService companySalesmanService;
    @Resource
    private CompanyService companyService;

    @GetMapping("/sale/info/company/update/log/filter")
    public BaseResponse filterCompanyUpdateLog(FilterCompanyUpdateLogRequest filterCompanyUpdateLogRequest,
                                               BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        if (StringUtils.isBlank(filterCompanyUpdateLogRequest.getCustomerId())) {
            filterCompanyUpdateLogRequest.setNoStatus("-1");
        }
        List<CompanyUpdateLogEntity> companyUpdateLogEntityList
                = companyUpdateLogService.filterCompanyUpdateLog(filterCompanyUpdateLogRequest);
        PageInfo<CompanyUpdateLogEntity> pageInfo;
        pageInfo = new PageInfo<>(companyUpdateLogEntityList);
        PageSimpleResponse<CompanyUpdateLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyUpdateLogEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/sale/info/companyUpdateLog")
    public BaseResponse getCompanyUpdateLogInfo(CompanyRequest companyRequest) {
        CompanyUpdateLogEntity companyUpdateLogEntity = companyUpdateLogService.findCompanyUpdateLog(companyRequest.getCustomerId());
        if (companyUpdateLogEntity != null) {
            return ResultVOUtils.success(companyUpdateLogEntity);
        }
        return ResultVOUtils.error(500, "空");
    }

    @GetMapping("/sale/info/getCompanyUpdateLog")
    public BaseResponse getCompanyUpdateLog(FilterCompanyUpdateLogRequest filterCompanyUpdateLogRequest) {
        filterCompanyUpdateLogRequest.setCustomerId(getCustomerId());
        List<CompanyUpdateLogEntity> companyUpdateLogEntities = companyUpdateLogService.getCompanyUpdateLog(filterCompanyUpdateLogRequest);
        PageInfo<CompanyUpdateLogEntity> pageInfo = new PageInfo<>(companyUpdateLogEntities);
        PageSimpleResponse<CompanyUpdateLogEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyUpdateLogEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/sale/info/getCompanyUpdateLogOne")
    public BaseResponse getCompanyUpdateLogOne(FilterCompanyUpdateLogRequest filterCompanyUpdateLogRequest) {
        filterCompanyUpdateLogRequest.setCustomerId(getCustomerId());
        filterCompanyUpdateLogRequest.setPage(1);
        filterCompanyUpdateLogRequest.setLimit(1);
        List<CompanyUpdateLogEntity> companyUpdateLogEntities = companyUpdateLogService.filterCompanyUpdateLog(filterCompanyUpdateLogRequest);
        if (companyUpdateLogEntities.size() > 0) {
            CompanyUpdateLogEntity companyUpdateLogEntity = companyUpdateLogEntities.get(0);
            return ResultVOUtils.success(companyUpdateLogEntity);
        }
        return ResultVOUtils.error(ResultEnum.DATA_NOT);
    }

    @PostMapping("/sale/info/addCompanyUpdateLog")
    public BaseResponse addCompanyUpdateLog(@RequestBody CompanyEntity companyEntity) {
        String jsonStr = JSON.toJSONString(companyEntity);
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        CompanyEntity tempCompanyEntity = JSON.toJavaObject(jsonObject, CompanyEntity.class);
        return ResultVOUtils.success("添加成功");
    }

    @PostMapping("/sale/info/applyUpdate")
    public BaseResponse applyUpdate(@RequestBody CompanyUpdateLogEntity companyUpdateLogEntity) {
        CompanyEntity companyEntity = new CompanyEntity();
        companyEntity.setCustomerId(companyUpdateLogEntity.getCustomerId());
        companyEntity.setAuditStatus("4");//申请变更
        companyService.modifyCompany(companyEntity);

        companyUpdateLogEntity.setCreate_time(new Date());
        companyUpdateLogEntity.setStatus("1");//未审核
        companyUpdateLogEntity.setLogType("2");//变更审核
        boolean b = companyUpdateLogService.insertCompanyUpdateLog(companyUpdateLogEntity);

        return ResultVOUtils.success("提交成功");
    }

    @PostMapping("/sale/info/applyUpdateManage")
    public BaseResponse applyUpdateManage(@RequestBody CompanyUpdateLogEntity companyUpdateLogEntity) {
        CompanyEntity companyEntity = new CompanyEntity();
        companyEntity.setCustomerId(companyUpdateLogEntity.getCustomerId());
        String auditStatus = "";
        if (companyUpdateLogEntity.getStatus().equals("2")) {
            auditStatus = "3";//允许变更
        } else {
            auditStatus = "1";//通过（拒绝变更5恢复为通过）
        }
        companyEntity.setAuditStatus(auditStatus);
        companyService.modifyCompany(companyEntity);

        companyUpdateLogEntity.setUpdate_time(new Date());
        boolean b = companyUpdateLogService.updateCompanyUpdateLog(companyUpdateLogEntity);

        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/insertCompanyUpdateLog")
    public BaseResponse insertCompanyUpdateLog(@RequestBody CompanyUpdateLogEntity companyUpdateLogEntity) {
        CompanySalesmanEntity companySalesmanEntity = new CompanySalesmanEntity();
//        companySalesmanEntity.setId(Integer.parseInt(String.valueOf(companyUpdateLogEntity.getSalesmanId())));
        companySalesmanEntity.setAuditStatus("4");
        companySalesmanEntity.setUpdate_time(new Date());
//        companySalesmanService.updateCompanySalesman(companySalesmanEntity);
        companyUpdateLogEntity.setCreate_time(new Date());
        companyUpdateLogEntity.setUpdate_time(new Date());
        companyUpdateLogEntity.setStatus("1");
        boolean b = companyUpdateLogService.insertCompanyUpdateLog(companyUpdateLogEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/updateCompanyUpdateLog")
    public BaseResponse updateCompanyUpdateLog(@RequestBody CompanyUpdateLogEntity companyUpdateLogEntity) {
        CompanySalesmanEntity companySalesmanEntity = new CompanySalesmanEntity();
//        companySalesmanEntity.setId(Integer.parseInt(String.valueOf(companyUpdateLogEntity.getSalesmanId())));
        String auditStatus = "";
        if (companyUpdateLogEntity.getStatus().equals("2")) {
            auditStatus = "3";//允许变更
        } else if (companyUpdateLogEntity.getStatus().equals("3")) {
            auditStatus = "1";//通过（拒绝变更5恢复为通过）
        }
        companySalesmanEntity.setAuditStatus(auditStatus);
        companySalesmanEntity.setUpdate_time(new Date());
//        companySalesmanService.updateCompanySalesman(companyCertEntity);
        companyUpdateLogEntity.setUpdate_time(new Date());
        FilterCompanyUpdateLogRequest filterCompanyUpdateLogRequest = new FilterCompanyUpdateLogRequest();
        filterCompanyUpdateLogRequest.setPage(1);
        filterCompanyUpdateLogRequest.setLimit(1);
        filterCompanyUpdateLogRequest.setCustomerId(String.valueOf(companyUpdateLogEntity.getCustomerId()));
        List<CompanyUpdateLogEntity> companyUpdateLogEntities =
                companyUpdateLogService.filterCompanyUpdateLog(filterCompanyUpdateLogRequest);
        if (companyUpdateLogEntities.size() > 0) {
            CompanyUpdateLogEntity tCompanyUpdateLogEntity = companyUpdateLogEntities.get(0);
            companyUpdateLogEntity.setId(tCompanyUpdateLogEntity.getId());
        }
        boolean b = companyUpdateLogService.updateCompanyUpdateLog(companyUpdateLogEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/sale/info/deleteCompanyUpdateLog")
    public BaseResponse deleteCompanyUpdateLog(@RequestBody DeleteCompanyFileRequest deleteCompanyFileRequest, BindingResult bindingResult) {
        boolean b = companyUpdateLogService.deleteById(deleteCompanyFileRequest.getId());
        return ResultVOUtils.success("成功");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
