package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.info.DeviceKeyEntity;
import com.glxp.sale.admin.req.info.FilterDeviceKeyRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.info.DeviceKeyService;
import com.glxp.sale.admin.util.DateUtil;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.security.MessageDigest;
import java.util.List;

@RestController
public class DeviceKeyController {

    @Resource
    DeviceKeyService deviceKeyService;

    @GetMapping("sale/device/registerList")
    public BaseResponse registerList(FilterDeviceKeyRequest filterDeviceKeyRequest) {
        List<DeviceKeyEntity> deviceKeyEntities = deviceKeyService.findDeviceKey(filterDeviceKeyRequest);
        PageInfo<DeviceKeyEntity> pageInfo;
        pageInfo = new PageInfo<>(deviceKeyEntities);
        PageSimpleResponse<DeviceKeyEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(deviceKeyEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("sale/device/check")
    public BaseResponse checkRegister(@RequestBody @Valid DeviceKeyEntity deviceKeyEntity,
                                      BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        deviceKeyEntity.setDate(DateUtil.getDate());
        deviceKeyService.updateDeviceKey(deviceKeyEntity);
        return ResultVOUtils.success("审核成功");

    }

    @PostMapping("/sale/info/registerKey")
    public BaseResponse registerKey(@RequestBody DeviceKeyEntity deviceKeyEntity) {
        String key = md5("glxp2020" + deviceKeyEntity.getImei());
        FilterDeviceKeyRequest filterDeviceKeyRequest = new FilterDeviceKeyRequest();
        filterDeviceKeyRequest.setImei(deviceKeyEntity.getImei());
        DeviceKeyEntity deviceKeyEntity1 = deviceKeyService.findDeviceByImei(filterDeviceKeyRequest);
        if (deviceKeyEntity1 == null) {
            deviceKeyEntity.setImeiKey(key);
            deviceKeyEntity.setDate(DateUtil.getDate());
            deviceKeyEntity.setIsCheck(0);
            deviceKeyService.insertDeviceKey(deviceKeyEntity);
        }
        if (deviceKeyEntity1.getIsCheck() != 0)
            return ResultVOUtils.success(key);
        else
            return ResultVOUtils.success("0");
    }

    public String md5(String password) {
        try {
            MessageDigest digest = MessageDigest.getInstance("MD5");
            byte[] bytes = digest.digest(password.getBytes());
            StringBuilder sb = new StringBuilder();
            for (byte b : bytes) {
                int c = b & 0xff; //负数转换成正数
                String result = Integer.toHexString(c); //把十进制的数转换成十六进制的书
                if (result.length() < 2) {
                    sb.append(0); //让十六进制全部都是两位数
                }
                sb.append(result);
            }
            return sb.toString(); //返回加密后的密文
        } catch (Exception ex) {
            ex.printStackTrace();
            return "";
        }
    }
}
