package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.info.SupplierEnterpriseBasicCertEntity;
import com.glxp.sale.admin.entity.info.SupplierEnterpriseBasicCertEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.info.CompanyRequest;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.FilterSuppliersEnterpriseBasicCertRequest;
import com.glxp.sale.admin.req.info.FilterSuppliersEnterpriseBasicCertRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.SuppliersEnterpriseBasicCertService;
import com.glxp.sale.admin.service.info.SuppliersEnterpriseBasicCertService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class SuppliersEnterpriseBasicCertController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private SuppliersEnterpriseBasicCertService suppliersEnterpriseBasicCertService;

    @GetMapping("/udiwms/info/suppliers/enterprise/basic/cert/filter")
    public BaseResponse filterCompanyCert(FilterSuppliersEnterpriseBasicCertRequest filterSuppliersEnterpriseBasicCertRequest,
                                          BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<SupplierEnterpriseBasicCertEntity> supplierEnterpriseBasicCertEntityList
                = suppliersEnterpriseBasicCertService.filterCompanyCert(filterSuppliersEnterpriseBasicCertRequest);
        PageInfo<SupplierEnterpriseBasicCertEntity> pageInfo;
        pageInfo = new PageInfo<>(supplierEnterpriseBasicCertEntityList);
        PageSimpleResponse<SupplierEnterpriseBasicCertEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(supplierEnterpriseBasicCertEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/info/suppliers/enterprise/basic/cert/companyCert")
    public BaseResponse getCompanyCertInfo(FilterSuppliersEnterpriseBasicCertRequest filterSuppliersEnterpriseBasicCertRequest) {
        SupplierEnterpriseBasicCertEntity supplierEnterpriseBasicCertEntity =
                suppliersEnterpriseBasicCertService.findCompanyCert(filterSuppliersEnterpriseBasicCertRequest.getEnterpriseId());
        if (supplierEnterpriseBasicCertEntity != null) {
            return ResultVOUtils.success(supplierEnterpriseBasicCertEntity);
        }
        return ResultVOUtils.error(500, "空");
    }

    @GetMapping("/udiwms/info/suppliers/enterprise/basic/cert/getCompanyCert")
    public BaseResponse getCompanyCert(FilterSuppliersEnterpriseBasicCertRequest filterSuppliersEnterpriseBasicCertRequest) {
//        filterSuppliersEnterpriseBasicCertRequest.setEnterpriseId(getCustomerId());
        List<SupplierEnterpriseBasicCertEntity> companyCertEntities = suppliersEnterpriseBasicCertService.getCompanyCert(filterSuppliersEnterpriseBasicCertRequest);
        PageInfo<SupplierEnterpriseBasicCertEntity> pageInfo = new PageInfo<>(companyCertEntities);
        PageSimpleResponse<SupplierEnterpriseBasicCertEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyCertEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/info/suppliers/enterprise/basic/cert/insertCompanyCert")
    public BaseResponse insertCompanyCert(@RequestBody SupplierEnterpriseBasicCertEntity supplierEnterpriseBasicCertEntity) {
        supplierEnterpriseBasicCertEntity.setCreate_time(new Date());
        supplierEnterpriseBasicCertEntity.setUpdate_time(new Date());
        boolean b = suppliersEnterpriseBasicCertService.insertCompanyCert(supplierEnterpriseBasicCertEntity);
        return ResultVOUtils.success("成功");
    }

    @PostMapping("/udiwms/info/suppliers/enterprise/basic/cert/updateCompanyCert")
    public BaseResponse updateCompanyCert(@RequestBody SupplierEnterpriseBasicCertEntity supplierEnterpriseBasicCertEntity) {

        boolean b = suppliersEnterpriseBasicCertService.updateCompanyCert(supplierEnterpriseBasicCertEntity);
        return ResultVOUtils.success("修改成功");
    }

    @PostMapping("/udiwms/info/suppliers/enterprise/basic/cert/deleteCompanyCert")
    public BaseResponse deleteCompanyCert(@RequestBody DeleteCompanyFileRequest deleteCompanyFileRequest, BindingResult bindingResult) {
        boolean b = suppliersEnterpriseBasicCertService.deleteById(deleteCompanyFileRequest.getId());
        String URL="d:/1s/udiwms/register/file/image3/" + deleteCompanyFileRequest.getFilePath();
        File file=new File(URL);
        if(file.exists()&&file.isFile()) {
            file.delete();
        }
        return ResultVOUtils.success("成功");
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
