package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.auth.CustomerContactEntity;
import com.glxp.sale.admin.entity.auth.CustomerDetailEntity;
import com.glxp.sale.admin.entity.info.CompanyEntity;
import com.glxp.sale.admin.entity.info.SupplierEnterpriseBasicCertEntity;
import com.glxp.sale.admin.entity.info.SuppliersEnterpriseBasicEntity;
import com.glxp.sale.admin.entity.info.SuppliersRegistrationBasicEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.auth.CustomerInfoFilterRequest;
import com.glxp.sale.admin.req.info.DeleteRequest;
import com.glxp.sale.admin.req.info.FilterSuppliersEnterpriseBasicCertRequest;
import com.glxp.sale.admin.req.info.SuppliersEnterpriseBasicRequest;
import com.glxp.sale.admin.req.info.SuppliersRegistrationBasicRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.auth.impl.CustomerContactService;
import com.glxp.sale.admin.service.info.SuppliersEnterpriseBasicCertService;
import com.glxp.sale.admin.service.info.SuppliersEnterpriseBasicService;
import com.glxp.sale.admin.service.info.SuppliersRegistrationBasicService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class SuppliersEnterpriseBasicController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private SuppliersEnterpriseBasicService suppliersEnterpriseBasicService;
    @Resource
    private SuppliersRegistrationBasicService suppliersRegistrationBasicService;
    @Resource
    private SuppliersEnterpriseBasicCertService suppliersEnterpriseBasicCertService;

    @GetMapping("/udiwms/info/suppliers/enterprise/basic/getCompanyInfo")
    public BaseResponse getCompanyInfo(SuppliersEnterpriseBasicRequest suppliersEnterpriseBasicRequest) {
        SuppliersEnterpriseBasicEntity suppliersEnterpriseBasicEntity = suppliersEnterpriseBasicService.findCompany(suppliersEnterpriseBasicRequest.getId());
        if (suppliersEnterpriseBasicEntity != null) {
            return ResultVOUtils.success(suppliersEnterpriseBasicEntity);
        }
        return ResultVOUtils.error(500, "企业信息为空");
    }

    @GetMapping("/udiwms/info/suppliers/enterprise/basic/getCompanyList")
    public BaseResponse getCompanyList(SuppliersEnterpriseBasicRequest suppliersEnterpriseBasicRequest) {
//        suppliersEnterpriseBasicRequest.setId(suppliersEnterpriseBasicRequest.getId());
        List<SuppliersEnterpriseBasicEntity> companyEntities = suppliersEnterpriseBasicService.getCompany(suppliersEnterpriseBasicRequest);
        PageInfo<SuppliersEnterpriseBasicEntity> pageInfo = new PageInfo<>(companyEntities);
        PageSimpleResponse<SuppliersEnterpriseBasicEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/info/suppliers/enterprise/basic/addCompany")
    public BaseResponse addCompany(@RequestBody SuppliersEnterpriseBasicEntity suppliersEnterpriseBasicEntity) {
        suppliersEnterpriseBasicEntity.setCreateDate(new Date());
        suppliersEnterpriseBasicEntity.setUpdateDate(new Date());
        boolean b = suppliersEnterpriseBasicService.insertCompany(suppliersEnterpriseBasicEntity);
        if (b) {
            return ResultVOUtils.success("添加成功");
        } else {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
    }

    @PostMapping("/udiwms/info/suppliers/enterprise/basic/modifyCompany")
    public BaseResponse modifyCompany(@RequestBody SuppliersEnterpriseBasicEntity suppliersEnterpriseBasicEntity) {
        suppliersEnterpriseBasicEntity.setUpdateDate(new Date());
        boolean b = suppliersEnterpriseBasicService.modifyCompany(suppliersEnterpriseBasicEntity);
        if (b) {
            return ResultVOUtils.success("修改成功");
        } else {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
    }

    @GetMapping("/udiwms/info/suppliers/enterprise/basic/delete")
    public BaseResponse deleteContact(DeleteRequest deleteRequest) {
        FilterSuppliersEnterpriseBasicCertRequest filterSuppliersEnterpriseBasicCertRequest =
                new FilterSuppliersEnterpriseBasicCertRequest();
        filterSuppliersEnterpriseBasicCertRequest.setEnterpriseId(Long.parseLong(deleteRequest.getId()));
        List<SupplierEnterpriseBasicCertEntity> supplierEnterpriseBasicCertEntityList =
                suppliersEnterpriseBasicCertService.getCompanyCert(filterSuppliersEnterpriseBasicCertRequest);
        for (SupplierEnterpriseBasicCertEntity supplierEnterpriseBasicCertEntity : supplierEnterpriseBasicCertEntityList) {
            if (StringUtils.isNotBlank(supplierEnterpriseBasicCertEntity.getFilePath())) {
                String URL="d:/1s/udiwms/register/file/image3/" + supplierEnterpriseBasicCertEntity.getFilePath();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
        }
        SuppliersRegistrationBasicRequest suppliersRegistrationBasicRequest =
                new SuppliersRegistrationBasicRequest();
        suppliersRegistrationBasicRequest.setEnterpriseId(Long.parseLong(deleteRequest.getId()));
        List<SuppliersRegistrationBasicEntity> suppliersRegistrationBasicEntityList =
                suppliersRegistrationBasicService.getRegistration(suppliersRegistrationBasicRequest);
        for (SuppliersRegistrationBasicEntity suppliersRegistrationBasicEntity : suppliersRegistrationBasicEntityList) {
            if (StringUtils.isNotBlank(suppliersRegistrationBasicEntity.getFilePath())) {
                String URL="d:/1s/udiwms/register/file/image4/" + suppliersRegistrationBasicEntity.getFilePath();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
            if (StringUtils.isNotBlank(suppliersRegistrationBasicEntity.getInstructions())) {
                String URL="d:/1s/udiwms/register/file/image4/" + suppliersRegistrationBasicEntity.getInstructions();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
        }
        boolean b = suppliersEnterpriseBasicService.deleteById(deleteRequest.getId());
        if (b)
            return ResultVOUtils.success("删除成功");
        else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
