package com.glxp.sale.admin.controller.info;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.info.SuppliersRegistrationBasicEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.req.info.DeleteCompanyFileRequest;
import com.glxp.sale.admin.req.info.DeleteRequest;
import com.glxp.sale.admin.req.info.SuppliersRegistrationBasicRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.auth.AuthAdminService;
import com.glxp.sale.admin.service.info.SuppliersRegistrationBasicService;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;

@RestController
public class SuppliersRegistrationBasicController {
    @Resource
    private AuthAdminService authAdminService;
    @Resource
    private SuppliersRegistrationBasicService suppliersRegistrationBasicService;

    @GetMapping("/udiwms/info/suppliers/registration/basic/getRegistrationInfo")
    public BaseResponse getRegistrationInfo(SuppliersRegistrationBasicRequest suppliersRegistrationBasicRequest) {
        SuppliersRegistrationBasicEntity suppliersRegistrationBasicEntity = suppliersRegistrationBasicService.findRegistration(suppliersRegistrationBasicRequest.getId());
        if (suppliersRegistrationBasicEntity != null) {
            return ResultVOUtils.success(suppliersRegistrationBasicEntity);
        }
        return ResultVOUtils.error(500, "企业信息为空");
    }

    @GetMapping("/udiwms/info/suppliers/registration/basic/getRegistrationList")
    public BaseResponse getRegistrationList(SuppliersRegistrationBasicRequest suppliersRegistrationBasicRequest) {
//        suppliersRegistrationBasicRequest.setId(suppliersRegistrationBasicRequest.getId());
        List<SuppliersRegistrationBasicEntity> companyEntities = suppliersRegistrationBasicService.getRegistration(suppliersRegistrationBasicRequest);
        PageInfo<SuppliersRegistrationBasicEntity> pageInfo = new PageInfo<>(companyEntities);
        PageSimpleResponse<SuppliersRegistrationBasicEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(companyEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/info/suppliers/registration/basic/addRegistration")
    public BaseResponse addRegistration(@RequestBody SuppliersRegistrationBasicEntity suppliersRegistrationBasicEntity) {
        suppliersRegistrationBasicEntity.setCreate_date(new Date());
        suppliersRegistrationBasicEntity.setUpdate_date(new Date());
        boolean b = suppliersRegistrationBasicService.insertRegistration(suppliersRegistrationBasicEntity);
        if (b) {
            return ResultVOUtils.success("添加成功");
        } else {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
    }

    @PostMapping("/udiwms/info/suppliers/registration/basic/modifyRegistration")
    public BaseResponse modifyRegistration(@RequestBody SuppliersRegistrationBasicEntity suppliersRegistrationBasicEntity) {
        suppliersRegistrationBasicEntity.setUpdate_date(new Date());
        if (StringUtils.isNotBlank(suppliersRegistrationBasicEntity.getNewFilePath())) {
            if (StringUtils.isNotBlank(suppliersRegistrationBasicEntity.getFilePath())) {
                String URL="d:/1s/udiwms/register/file/image4/" + suppliersRegistrationBasicEntity.getFilePath();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
            suppliersRegistrationBasicEntity.setFilePath(suppliersRegistrationBasicEntity.getNewFilePath());
        }
        if (StringUtils.isNotBlank(suppliersRegistrationBasicEntity.getNewInstructions())) {
            if (StringUtils.isNotBlank(suppliersRegistrationBasicEntity.getInstructions())) {
                String URL="d:/1s/udiwms/register/file/image4/" + suppliersRegistrationBasicEntity.getInstructions();
                File file=new File(URL);
                if(file.exists()&&file.isFile()) {
                    file.delete();
                }
            }
            suppliersRegistrationBasicEntity.setInstructions(suppliersRegistrationBasicEntity.getNewInstructions());
        }
        boolean b = suppliersRegistrationBasicService.modifyRegistration(suppliersRegistrationBasicEntity);
        if (b) {
            return ResultVOUtils.success("修改成功");
        } else {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
    }

    @GetMapping("/udiwms/info/suppliers/registration/basic/delete")
    public BaseResponse delete(DeleteCompanyFileRequest deleteCompanyFileRequest) {
        if (StringUtils.isNotBlank(deleteCompanyFileRequest.getFilePath())) {
            String URL="d:/1s/udiwms/register/file/image4/" + deleteCompanyFileRequest.getFilePath();
            File file=new File(URL);
            if(file.exists()&&file.isFile()) {
                file.delete();
            }
        }
        if (StringUtils.isNotBlank(deleteCompanyFileRequest.getFilePath2())) {
            String URL="d:/1s/udiwms/register/file/image4/" + deleteCompanyFileRequest.getFilePath2();
            File file=new File(URL);
            if(file.exists()&&file.isFile()) {
                file.delete();
            }
        }
        boolean b = suppliersRegistrationBasicService.deleteById(deleteCompanyFileRequest.getId());
        if (b)
            return ResultVOUtils.success("删除成功");
        else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }

    public String getCustomerId() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new JsonException(ResultEnum.NOT_NETWORK);
        }
        HttpServletRequest request = attributes.getRequest();
        String userId = request.getHeader("ADMIN_ID");
        AuthAdmin authAdmin = authAdminService.findById(Long.parseLong(userId));
        return authAdmin.getCustomerId() + "";
    }
}
