package com.glxp.sale.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.sale.admin.controller.inout.utils.InoutUtils;
import com.glxp.sale.admin.entity.basic.BussinessTypeEntity;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.entity.inout.ErpOrderEntity;
import com.glxp.sale.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.sale.admin.entity.inout.OrderEntity;
import com.glxp.sale.admin.entity.inout.WarehouseEntity;
import com.glxp.sale.admin.entity.udid.UdiEntity;
import com.glxp.sale.admin.httpclient.ErpOrderClient;
import com.glxp.sale.admin.req.inout.*;
import com.glxp.sale.admin.req.param.SystemParamConfigRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.basic.BussinessTypeService;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.inout.CodesService;
import com.glxp.sale.admin.service.inout.OrderDetailService;
import com.glxp.sale.admin.service.inout.OrderService;
import com.glxp.sale.admin.thread.IoTransInoutService;
import com.glxp.sale.admin.util.FilterUdiUtils;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class OrderController {

    @Resource
    private OrderService orderService;
    @Resource
    private CodesService codesService;
    @Resource
    private OrderDetailService orderDetailService;
    @Resource
    private UdiInfoService udiInfoService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Value("${UCLOD_SERVER_URL}")
    private String ucloudUrl;
    @Resource
    IoTransInoutService ioTransInoutService;

    @GetMapping("udiwms/inout/order/getUdi")
    public BaseResponse getUdi(@RequestParam("code") String code) {
        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null) return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        return ResultVOUtils.success(udiEntity);
    }

    @GetMapping("udiwms/inout/order/status")
    public BaseResponse listOrderByStatus(OrderFilterRequest orderFilterRequest) {
        List<OrderEntity> orderEntityList = orderService.filterList(orderFilterRequest);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("udiwms/inout/order/error")
    public BaseResponse listOrderError(OrderFilterRequest orderFilterRequest) {
//        orderFilterRequest.setStatus(ConstantStatus.ORDER_STATUS_FAIL);
//        orderFilterRequest.setStatusOrOne(ConstantStatus.ORDER_STATUS_TEMP_SAVE);
        if (StringUtils.isBlank(orderFilterRequest.getCustomerId()))
            return ResultVOUtils.success();
        if (orderFilterRequest.getCustomerId().equals("110"))
            orderFilterRequest.setCustomerId(null);
//        List<OrderEntity> orderEntityList = orderService.filterListOr(orderFilterRequest);
        List<OrderEntity> orderEntityList = orderService.filterList(orderFilterRequest);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("udiwms/inout/order/repeat")
    public BaseResponse listOrderRepeat(OrderQueryRequest orderQueryRequest) {
        List<OrderEntity> orderEntityList = orderService.listOrderRepeat(orderQueryRequest);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("udiwms/inout/order/delete")
    public BaseResponse deleteOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        orderService.deleteById(id);
        orderDetailService.deleteByOrderId(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/inout/order/updateUnit")
    public BaseResponse updateUnit(@RequestBody @Valid OrderEntity orderEntity,
                                   BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        boolean b = orderService.updateUnit(orderEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        b = codesService.updateUnit(orderEntity.getId(), orderEntity.getFromCorpId(), orderEntity.getFromCorp());
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("添加成功");
    }

    @GetMapping("/udiwms/inout/order/filter")
    public BaseResponse filterOrder(OrderFilterRequest orderFilterRequest,
                                    BindingResult bindingResult) {

        if (StringUtils.isBlank(orderFilterRequest.getCustomerId())) return ResultVOUtils.success();
        if (orderFilterRequest.getCustomerId().equals("110")) orderFilterRequest.setCustomerId(null);
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<OrderEntity> orderEntityList;
        if (orderFilterRequest.getStatus() == null || orderFilterRequest.getStatus() == 0) {
            orderFilterRequest.setStatus(4);
            orderFilterRequest.setStatusOrOne(3);
        }
        if (orderFilterRequest.getCode() != null && !orderFilterRequest.getCode().equals("")) {
            List<WarehouseEntity> warehouseEntityList = codesService.findByCode(orderFilterRequest.getCode());
            orderEntityList = orderService.filterListByCode(warehouseEntityList);
        } else {
            orderEntityList = orderService.filterListOr(orderFilterRequest);
        }
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);//TODO 已完成单据列表
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/inout/order/filterCode")
    public BaseResponse filterOrderByCode(OrderFilterByCodeRequest orderFilterByCodeRequest,
                                          BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<WarehouseEntity> warehouseEntityList = codesService.findByCode(orderFilterByCodeRequest.getCode());
        List<OrderEntity> orderEntityList = orderService.filterListByCode(warehouseEntityList);
        PageInfo<OrderEntity> pageInfo;
        pageInfo = new PageInfo<>(orderEntityList);
        PageSimpleResponse<OrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(orderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/inout/order/updateExportStatus")
    public BaseResponse updateExportStatus(@RequestBody UpdateExportStatusRequest updateExportStatusRequest,
                                           BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = orderService.updateExportStatus(updateExportStatusRequest.getOrderId(), updateExportStatusRequest.getExportStatus());
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }
        return ResultVOUtils.success("更新成功");
    }

    @PostMapping("/warehouse/inout/order/mergeOrder")//TODO 合并订单
    public BaseResponse mergeOrder(@RequestBody OrderMergeRequest orderMergeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        List<String> orderList = orderMergeRequest.getOrderList();

        SystemParamConfigRequest systemParamConfigRequest = new SystemParamConfigRequest();
        systemParamConfigRequest.setParamKey("forbidSameErp");
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey(systemParamConfigRequest);
        if (systemParamConfigEntity == null || systemParamConfigEntity.getParamValue().equals("0")) {
            List<ErpOrderEntity> mergeList = new ArrayList<>();
            if (orderList != null && orderList.size() > 1) {
                for (String orderId : orderList) {
                    FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
                    filterErpOrderRequest.setOrderId(orderId);
                    List<ErpOrderEntity> erpOrderEntities = orderDetailService.filterAllMyErpOrder(filterErpOrderRequest);
                    if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                        mergeList.addAll(erpOrderEntities);
                    }
                    orderDetailService.deleteByOrderId(orderId);

                }
                String mainOrder = orderList.get(0);
                for (int i = 1; i < orderList.size(); i++) {
                    codesService.updateOrderId(orderList.get(i), mainOrder);
                    orderService.deleteByOrderId(orderList.get(i));
                }
                OrderEntity mainOrderEntity = orderService.findById(mainOrder);
                BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(mainOrderEntity.getAction());
                List<String> docids = getDocids(mergeList);
                String docidStr = "";
                if (docids != null && docids.size() > 0) {
                    for (String temp : docids) {
                        docidStr = docidStr + "," + temp;
                    }
                    docidStr = docidStr.substring(1);
                }
                mainOrderEntity.setErpFk(docidStr);
                orderService.updateOrder(mainOrderEntity);
                List<ErpOrderEntity> erpOrderEntities = new ErpOrderClient(ucloudUrl).getErpOrder(docids, mainOrderEntity.getAction());
                if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                    List<ErpOrderEntity> returnOrders = new ContrastErpUtil().transErp(erpOrderEntities, udiRelevanceService, mainOrder);
                    orderDetailService.insertErpOrders(returnOrders);
                    List<WarehouseEntity> warehouseEntityList = codesService.findByReceiptId(mainOrder);
                    new ContrastErpUtil().contrastErp(udiInfoService, udiRelevanceService, orderDetailService, orderService, warehouseEntityList,
                            systemParamConfigService, bussinessTypeEntity);
                } else {
                    orderService.updateContrastStatus(mainOrder, "error");
                    orderService.updateRemark(mainOrder, "U8Cloud获取ERP单失败");
                    return ResultVOUtils.error(500, "U8Cloud获取ERP单失败");
                }
            } else {
                return ResultVOUtils.error(ResultEnum.DATA_ERROR);
            }
        } else {
            if (orderList != null && orderList.size() > 1) {
                for (String orderId : orderList) {
                    FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
                    filterErpOrderRequest.setOrderId(orderId);
                    List<ErpOrderEntity> erpOrderEntities = orderDetailService.filterAllMyErpOrder(filterErpOrderRequest);
                    if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                        for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                            if (erpOrderEntity.getErpOrderId() != null) {
                                return ResultVOUtils.error(500, "已校验过订单不能合并！");
                            }
                        }
                    }
                    orderDetailService.deleteByOrderId(orderId);
                }
                String mainOrder = orderList.get(0);
                for (int i = 1; i < orderList.size(); i++) {
                    codesService.updateOrderId(orderList.get(i), mainOrder);
                    orderService.deleteByOrderId(orderList.get(i));
                }
                new Thread(() -> {
                    List<WarehouseEntity> datas = codesService.findByReceiptId(mainOrder);
                    new InoutUtils().generateConstrastDetail(udiInfoService, udiRelevanceService, orderDetailService, datas);
                    ioTransInoutService.printOrder(mainOrder);
                }).start();
            }
        }

        return ResultVOUtils.success("合并成功,后台正在数据转换，请稍后刷新重试！");
    }


    public List<String> getDocids(List<ErpOrderEntity> mergeList) {
        List<String> docids = new ArrayList<>();
        Map<String, List<ErpOrderEntity>> mapErps = mergeList.stream().collect(Collectors.groupingBy(ErpOrderEntity::getErpOrderId));
        for (String key : mapErps.keySet()) {
            docids.add(key);
        }
        return docids;
    }


}
