package com.glxp.sale.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.sale.admin.entity.auth.AuthAdmin;
import com.glxp.sale.admin.entity.basic.BussinessTypeEntity;
import com.glxp.sale.admin.entity.info.CompanyEntity;
import com.glxp.sale.admin.entity.inout.*;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.entity.thrsys.ThrOrderDetailEntity;
import com.glxp.sale.admin.entity.thrsys.ThrOrderEntity;
import com.glxp.sale.admin.exception.JsonException;
import com.glxp.sale.admin.httpclient.ErpOrderClient;
import com.glxp.sale.admin.req.inout.*;
import com.glxp.sale.admin.req.param.SystemParamConfigRequest;
import com.glxp.sale.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.sale.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.inout.ErpOrderResponse;
import com.glxp.sale.admin.res.inout.PurchaseinResponse;
import com.glxp.sale.admin.service.basic.BussinessTypeService;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.info.CompanyService;
import com.glxp.sale.admin.service.inout.*;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.thrsys.ThrOrderDetailService;
import com.glxp.sale.admin.service.thrsys.ThrOrderService;
import com.glxp.sale.admin.thread.InvDlErpStockService;
import com.glxp.sale.admin.thread.IoTransInoutService;
import com.glxp.sale.admin.util.DateUtil;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class OrderDetailController {

    @Resource
    OrderDetailService orderDetailService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    CodesService codesService;
    @Resource
    OrderService orderService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private InvDlErpStockService dlErpStockService;


    @Resource
    private IoTransInoutService transInoutService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;

    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    private ThrOrderDetailService thrOrderDetailService;

    @Resource
    private StockOrderService stockOrderService;
    @Resource
    private StockOrderDetailService stockOrderDetailService;

    @Resource
    private CompanyService companyService;
    @Resource
    private BussinessTypeService bussinessTypeService;

    @GetMapping("/udiwms/erpOrder/filter")
    public BaseResponse filterErpOrder(FilterOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }


        FilterThrOrderRequest filterThrOrderRequest = new FilterThrOrderRequest();
        BeanUtils.copyProperties(filterErpOrderRequest, filterThrOrderRequest);
        filterThrOrderRequest.setPage(filterErpOrderRequest.getPage());
        filterThrOrderRequest.setLimit(filterErpOrderRequest.getLimit());

        if (filterErpOrderRequest.getCustomerId() != null) {
            CompanyEntity companyEntity = companyService.findCompany(Long.parseLong(filterErpOrderRequest.getCustomerId()));
            filterThrOrderRequest.setUnitIdFk(companyEntity.getUnitIdFk());
        }


        List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
        List<ThrOrderEntity> data = thrOrderService.filterThrOrder(filterThrOrderRequest);
        if (data != null && data.size() > 0) {
            for (ThrOrderEntity thrOrderEntity : data) {
                FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
                filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
                List<ThrOrderDetailEntity> thrOrderDetailEntities =
                        thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
                ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                BeanUtils.copyProperties(thrOrderEntity, erpOrderResponse);
                List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                    for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                        ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                        BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                        subErpOrders.add(subErpOrder);
                    }
                }
                erpOrderResponse.setSubErpOrders(subErpOrders);
                erpOrderResponses.add(erpOrderResponse);
            }
        }
        PageInfo<ThrOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(data);
        PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpOrderResponses);
        return ResultVOUtils.success(pageSimpleResponse);
//        }
    }

    @GetMapping("/udiwms/erpOrder/addStockOrder")
    public BaseResponse addStockOrder(FilterOrderRequest filterErpOrderRequest,
                                      BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ThrOrderEntity thrOrderEntity = thrOrderService.findById(String.valueOf(filterErpOrderRequest.getId()));
        if (thrOrderEntity == null) {
            return ResultVOUtils.error(ResultEnum.DATA_NOT);
        }
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        BeanUtils.copyProperties(thrOrderEntity, stockOrderEntity);

        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(thrOrderEntity.getBillType());
        stockOrderEntity.setBillType(bussinessTypeEntity.getLocalAction());
        stockOrderEntity.setId(null);
        stockOrderEntity.setStatus(ConstantStatus.SORDER_DRAFT);//未提交
        stockOrderEntity.setSourceType(ConstantStatus.ST_THIRD_SEL);//外部提交
        stockOrderEntity.setPrintStatus(ConstantStatus.SORDER_PRINT_UN);//未打印
        stockOrderEntity.setBilldate(DateUtil.formatDate(new Date()));
        boolean b = stockOrderService.insertStockOrder(stockOrderEntity);
        if (b) {
            FilterThrOrderDetailRequest filterThrOrderDetailRequest = new FilterThrOrderDetailRequest();
            filterThrOrderDetailRequest.setOrderIdFk(thrOrderEntity.getId() + "");
            List<ThrOrderDetailEntity> thrOrderDetailEntities =
                    thrOrderDetailService.filterThrOrderDetailDetail(filterThrOrderDetailRequest);
            if (thrOrderDetailEntities != null && thrOrderDetailEntities.size() > 0) {
                List<StockOrderDetailEntity> stockOrderDetailEntityList = new ArrayList<>();
                for (ThrOrderDetailEntity thrOrderDetailEntity : thrOrderDetailEntities) {
                    StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
                    BeanUtils.copyProperties(thrOrderDetailEntity, stockOrderDetailEntity);
                    stockOrderDetailEntity.setCount(thrOrderDetailEntity.getCount() + "");
                    stockOrderDetailEntity.setReCount(thrOrderDetailEntity.getReCount() + "");
                    stockOrderDetailEntity.setOrderIdFk(stockOrderEntity.getId());
                    stockOrderDetailEntity.setSweepCount(0 + "");
                    stockOrderDetailEntityList.add(stockOrderDetailEntity);
                }
                stockOrderDetailService.insertStockOrderDetails(stockOrderDetailEntityList);
            }
        }

        return ResultVOUtils.success();
    }


    @PostMapping("/udiwms/erpOrder/updateBind")
    public BaseResponse updateBind(@RequestBody ErpOrderEntity erpOrderEntity, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = orderDetailService.updateErpOrder(erpOrderEntity);
        if (b) {
            return ResultVOUtils.success("绑定成功");
        } else {
            return ResultVOUtils.error(500, "绑定失败");
        }
    }


    @GetMapping("/warehouse/getMyErpOrders")
    public BaseResponse getMyErpOrders(FilterErpOrderRequest filterErpOrderRequest,
                                       BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getErpOrderId() == null || filterErpOrderRequest.getErpOrderId().equals("")) {
            filterErpOrderRequest.setErpOrderId("null");
        }
        List<ErpOrderEntity> erpInfoEntityList = orderDetailService.filterMyErpDetailOrder(filterErpOrderRequest);
        PageInfo<ErpOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(erpInfoEntityList);
        PageSimpleResponse<ErpOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/deleteErpOrder")
    public BaseResponse deleteErpOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = orderDetailService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }

    @GetMapping(value = "/warehouse/inout/order/detail")//TODO 订单详情
    public BaseResponse orderDetail(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            filterErpOrderRequest.setOrderId("1");
        }
        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = orderDetailService.filterMyErpOrder(filterErpOrderRequest);
        return ResultVOUtils.success(erpOrderEntities);
    }

    @GetMapping(value = "/warehouse/inout/order/uploadOrder")
    public BaseResponse uploadOrder(FilterErpOrderRequest filterErpOrderRequest,
                                    BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (filterErpOrderRequest.getOrderId() == null || filterErpOrderRequest.getOrderId().equals("")) {
            return ResultVOUtils.error(500, "上传失败");
        }

        filterErpOrderRequest.setPage(1);
        filterErpOrderRequest.setLimit(100);
        List<ErpOrderEntity> erpOrderEntities = orderDetailService.filterMyErpOrder(filterErpOrderRequest);
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        if (erpOrderEntities != null) {
            for (int i = 0; i < erpOrderEntities.size(); i++) {
                if (erpOrderEntities.get(i).getErpOrderId() == null) {
                    return ResultVOUtils.error(500, "批次号不匹配，请检查后修改！");
                }

                if (erpOrderEntities.get(i).getErpOrderId().contains(",")) {
                    return ResultVOUtils.error(500, "无法修改合并单据！");
                }

                if (erpOrderEntities.get(i).getCount() > erpOrderEntities.get(i).getErpCount()) {
                    return ResultVOUtils.error(500, "扫码数量大于实际数量,无法修改！");
                }
            }
        }
        SystemParamConfigRequest systemParamConfigRequest = new SystemParamConfigRequest();
        systemParamConfigRequest.setParamKey("erp_order_modify");
        SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey(systemParamConfigRequest);
        if (systemParamConfigEntity == null)
            return ResultVOUtils.error(500, "ERP出入库单据修改接口未定义");
        String url = systemParamConfigEntity.getParamValue();
        Map<String, List<ErpOrderEntity>> listMap = erpOrderEntities.stream().collect(Collectors.groupingBy(ErpOrderEntity::getErpOrderId));
        BaseResponse baseResponse = new ErpOrderClient().postBill(listMap, orderEntity.getAction(), url);
        if (baseResponse != null && baseResponse.getCode() == 20000) {
            orderService.updateExportStatus(orderEntity.getId(), ConstantStatus.ORDER_EXPORT_ED);
            return ResultVOUtils.success("修改成功");
        } else {
            if (baseResponse != null) {
                return ResultVOUtils.error(500, baseResponse.getMessage());
            }
            return ResultVOUtils.error(500, "修改失败");
        }
    }


    @GetMapping("/warehouse/dlErpStock1")
    public BaseResponse dlErpStock() {
        dlErpStockService.dlErpStock();
        return ResultVOUtils.success("后台已自动更新，请稍后刷新查看！");
    }

    @PostMapping("/warehouse/selectErpOrder")
    public BaseResponse selectErpOrder(@RequestBody ImportErpOrderRequest importErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ErpOrderResponse> docids = importErpOrderRequest.getPurchaseinResponses();
        String docidStr = "";
        if (docids != null && docids.size() > 0) {
            for (ErpOrderResponse temp : docids) {
                docidStr = docidStr + "," + temp.getBillNo();
            }
            docidStr = docidStr.substring(1);
        }
        List<ErpOrderEntity> erpOrderEntities = new ArrayList<>();
        if (docids != null && docids.size() > 0) {
            for (ErpOrderResponse purchaseinResponse : docids) {
                for (ErpOrderResponse.SubErpOrder subPurchase : purchaseinResponse.getSubErpOrders()) {
                    ErpOrderEntity erpOrderEntity = new ErpOrderEntity();
                    erpOrderEntity.setErpOrderId(purchaseinResponse.getBillNo());
                    erpOrderEntity.setCompanyid(purchaseinResponse.getCorpId());
                    erpOrderEntity.setCompanyname(purchaseinResponse.getCorpName());
                    erpOrderEntity.setCredate(purchaseinResponse.getBilldate());
                    erpOrderEntity.setGoodsid(subPurchase.getProductId());
                    erpOrderEntity.setGoodsname(subPurchase.getProductName());
                    erpOrderEntity.setBatchNo(subPurchase.getBatchNo());
                    erpOrderEntity.setErpCount(Math.abs(subPurchase.getCount()));
                    erpOrderEntity.setReCount(Math.abs(subPurchase.getReCount()));
                    erpOrderEntity.setGoodsunit(subPurchase.getSpec());
                    erpOrderEntities.add(erpOrderEntity);
                }
            }
        }
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            ErpOrderEntity temp = erpOrderEntities.get(0);
            for (int i = 0; i < erpOrderEntities.size(); i++) {
                if (!temp.getCompanyname().equals(erpOrderEntities.get(i).getCompanyname())) {
                    return ResultVOUtils.error(500, "选择Erp订单往来单位不一致！");
                }
            }

            OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
            orderFilterRequest.setId(importErpOrderRequest.getOrderId());
            OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());
            ErpOrderEntity exitErp = new ContrastErpUtil().vailErpExit(orderDetailService, erpOrderEntities, importErpOrderRequest.getOrderId());
            if (exitErp != null) {
                String errMsg = "所选ERP单" + exitErp.getErpOrderId() + "已被其他出入库单" + exitErp.getOrderIdFk() + "绑定";
                orderEntity.setRemark(errMsg);
                orderService.updateOrder(orderEntity);
                return ResultVOUtils.error(500, errMsg);
            } else {
                orderEntity.setErpFk(docidStr);
                orderService.updateOrder(orderEntity);
            }

            orderDetailService.deleteByOrderId(importErpOrderRequest.getOrderId());
            List<ErpOrderEntity> returnOrders = new ContrastErpUtil().transErp(erpOrderEntities, udiRelevanceService, importErpOrderRequest.getOrderId());
            orderDetailService.insertErpOrders(returnOrders);
            List<WarehouseEntity> warehouseEntityList = codesService.findByReceiptId(importErpOrderRequest.getOrderId());
            new ContrastErpUtil().contrastErp(udiInfoService, udiRelevanceService, orderDetailService, orderService, warehouseEntityList, systemParamConfigService, bussinessTypeEntity);


        } else {
            return ResultVOUtils.error(500, "ERP订单不能为空");
        }

        return ResultVOUtils.success("导入成功");
    }


    @PostMapping("/warehouse/repeatCheck")//TODO 重新校验
    public BaseResponse repeatCheck(@RequestBody FilterErpOrderRequest filterErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        transInoutService.repeatCheck(orderEntity.getId());
        return ResultVOUtils.success("后台已开始校验，请稍后刷新重试！");
    }

    @PostMapping("/warehouse/repeatPrintCheck")//TODO 重新校验
    public BaseResponse repeatPrintCheck(@RequestBody FilterErpOrderRequest filterErpOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(filterErpOrderRequest.getOrderId());
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        transInoutService.repeatPrint(orderEntity);
        return ResultVOUtils.success("后台已开始校验，请稍后刷新重试！");
    }
}
