package com.glxp.sale.admin.controller.inout;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.constant.IncodeStatus;
import com.glxp.sale.admin.controller.inout.inventory.InvPrintController;
import com.glxp.sale.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.sale.admin.controller.inout.utils.InoutUtils;
import com.glxp.sale.admin.entity.basic.UdiInfoEntity;
import com.glxp.sale.admin.entity.basic.StockPrintEntity;
import com.glxp.sale.admin.entity.info.CompanyEntity;
import com.glxp.sale.admin.entity.inout.*;
import com.glxp.sale.admin.entity.inventory.InCodeLogEntity;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.entity.thrsys.ThrOrderDetailEntity;
import com.glxp.sale.admin.entity.thrsys.ThrOrderEntity;
import com.glxp.sale.admin.entity.udid.UdiEntity;
import com.glxp.sale.admin.httpclient.ErpOrderClient;
import com.glxp.sale.admin.req.inout.*;
import com.glxp.sale.admin.req.inventory.PostStockPrintRequest;
import com.glxp.sale.admin.req.param.SystemParamConfigRequest;
import com.glxp.sale.admin.req.thrsys.FilterThrOrderDetailRequest;
import com.glxp.sale.admin.req.thrsys.FilterThrOrderRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.inout.ErpOrderResponse;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.info.CompanyService;
import com.glxp.sale.admin.service.inout.*;
import com.glxp.sale.admin.service.inventory.InCodeLogService;
import com.glxp.sale.admin.service.inventory.InvStockPrintService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.util.*;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@RestController
public class StockOrderController {

    @Resource
    private StockOrderService stockOrderService;
    @Resource
    private CodesService codesService;
    @Resource
    private StockOrderDetailService stockOrderDetailService;
    @Resource
    private StockOrderDetailTempService stockOrderDetailTempService;
    @Resource
    private StockQRCodeTextService stockQRCodeTextService;
    @Resource
    private UdiInfoService udiInfoService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private SystemParamConfigService systemParamConfigService;

    @Value("${STOCK_PRINT_POST}")
    private String stpUrl;

    @Resource
    private InvPrintController invPrintController;
    @Resource
    InvStockPrintService stockPrintService;
    @Resource
    InCodeLogService inCodeLogService;
    @Resource
    CompanyService companyService;

    @GetMapping("/udiwms/stock/order/error")
    public BaseResponse listStockOrderNoDealWith(StockOrderFilterRequest stockOrderFilterRequest) {
        List<StockOrderEntity> stockOrderEntityList = stockOrderService.filterListOr(stockOrderFilterRequest);
        PageInfo<StockOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(stockOrderEntityList);
        PageSimpleResponse<StockOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockOrderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @PostMapping("/udiwms/stock/order/postDealWith")
    public BaseResponse postDealWith(@RequestBody StockOrderPostRequest stockOrderPostRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        BeanUtils.copyProperties(stockOrderPostRequest, stockOrderEntity);
        stockOrderEntity.setStatus(ConstantStatus.SORDER_DRAFT);//未处理
        stockOrderEntity.setPrintStatus(ConstantStatus.SORDER_PRINT_UN);//未打印
        stockOrderEntity.setSourceType(ConstantStatus.ST_THIRD_SEL);//外部提交
//        stockOrderEntity.setId(String.valueOf(new Date().getTime()));
        stockOrderEntity.setBilldate(DateUtil.formatDate(new Date()));
        boolean b = stockOrderService.insertStockOrder(stockOrderEntity);
        if (b) {
            List<StockOrderDetailEntity> stockOrderDetailEntityList = stockOrderPostRequest.getSubErpOrders();
            for (StockOrderDetailEntity stockOrderDetailEntity : stockOrderDetailEntityList) {
                stockOrderDetailEntity.setOrderIdFk(stockOrderEntity.getId());
                stockOrderDetailEntity.setSweepCount(0 + "");
            }
            stockOrderDetailTempService.insertStockOrderDetailTemps(stockOrderDetailEntityList);
        }

        return ResultVOUtils.success("成功");
    }

    @PostMapping("/udiwms/stock/order/insertWeb")
    public BaseResponse insertWeb(@RequestBody StockOrderPostRequest stockOrderPostRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        boolean noRepeatBillNo = false;
        boolean newBillNo = false;
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        while (!noRepeatBillNo) {
            if (newBillNo) {
                String billNo = StockOrderBillNoUtils.getBillNo(stockOrderPostRequest.getBillType());
                stockOrderPostRequest.setBillNo(billNo);
            }
            if (StringUtils.isBlank(stockOrderPostRequest.getBillNo())) {
                newBillNo = true;
                continue;
            }
            stockOrderFilterRequest.setBillNo(stockOrderPostRequest.getBillNo());
            if (stockOrderService.findOne(stockOrderFilterRequest) == null) {
                noRepeatBillNo = true;
                newBillNo = false;
            } else {
                newBillNo = true;
            }
        }

        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        BeanUtils.copyProperties(stockOrderPostRequest, stockOrderEntity);

        stockOrderEntity.setStatus(
                StringUtils.isBlank(stockOrderEntity.getStatus()) ? 501 + ""//未提交
                        : stockOrderEntity.getStatus());
        stockOrderEntity.setPrintStatus(ConstantStatus.SORDER_PRINT_UN);//未打印
        stockOrderEntity.setSourceType(ConstantStatus.ST_WEB_ADD);//本地网页新增
        boolean b = false;
        if (stockOrderEntity.getStatus().equals("502")) {//已提交
            SystemParamConfigEntity systemParamConfigEntity =
                    systemParamConfigService.selectByParamKey("stock_order_new_submit_type");
            if (systemParamConfigEntity == null || systemParamConfigEntity.getParamValue().equals("0")) {
                stockOrderEntity.setStatus("201");//未配货
            } else {
                stockOrderEntity.setStatus("202");//已配货
            }
        }
        if (StringUtils.isBlank(stockOrderEntity.getId())) {

            CompanyEntity companyEntity = companyService.findCompany(Long.parseLong(stockOrderPostRequest.getCustomerId()));
            stockOrderEntity.setCorpId(companyEntity.getUnitIdFk());
            stockOrderEntity.setCorpName(companyEntity.getCompanyName());
            stockOrderEntity.setUnitIdFk(companyEntity.getUnitIdFk());
            stockOrderEntity.setCustomerId(stockOrderPostRequest.getCustomerId());
            stockOrderEntity.setBilldate(DateUtil.formatDate(new Date()));
            b = stockOrderService.insertStockOrder(stockOrderEntity);
        } else {
            stockOrderService.updateById(stockOrderEntity);
        }
        if (stockOrderPostRequest.getSubErpOrders() != null) {
            List<StockOrderDetailEntity> stockOrderDetailEntityList = stockOrderPostRequest.getSubErpOrders();
            for (StockOrderDetailEntity stockOrderDetailEntity : stockOrderDetailEntityList) {
                stockOrderDetailEntity.setOrderIdFk(
                        StringUtils.isBlank(stockOrderDetailEntity.getOrderIdFk()) ?
                                stockOrderEntity.getId() : stockOrderDetailEntity.getOrderIdFk());

                if (StringUtils.isBlank(stockOrderDetailEntity.getId())) {
                    stockOrderDetailEntity.setReCount(
                            StringUtils.isBlank(stockOrderDetailEntity.getReCount()) ? "0" : stockOrderDetailEntity.getReCount());
                    stockOrderDetailEntity.setCount(
                            StringUtils.isBlank(stockOrderDetailEntity.getCount()) ? "0" : stockOrderDetailEntity.getCount());
                    stockOrderDetailEntity.setSweepCount(
                            StringUtils.isBlank(stockOrderDetailEntity.getSweepCount()) ? "0" : stockOrderDetailEntity.getSweepCount());

                    stockOrderDetailService.insertStockOrderDetail(stockOrderDetailEntity);
                } else {
                    stockOrderDetailService.updateById(stockOrderDetailEntity);
                }
            }
//            stockOrderDetailService.insertStockOrderDetails(stockOrderDetailEntityList);
        }

        return ResultVOUtils.success("成功");
    }

    @PostMapping("/udiwms/stock/order/delete")
    public BaseResponse deleteOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        stockOrderService.deleteById(id);
        stockOrderDetailService.deleteByOrderIdFk(id);
        stockOrderDetailTempService.deleteByOrderIdFk(id);
        return ResultVOUtils.success("删除成功");
    }

    @PostMapping("/udiwms/stock/order/error/delete")
    public BaseResponse deleteErrorOrder(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();

        stockOrderService.deleteById(id);
        stockOrderDetailTempService.deleteByOrderIdFk(id);
        return ResultVOUtils.success("删除成功");
    }

    @GetMapping("/udiwms/stock/order/filter")
    public BaseResponse filterOrder(StockOrderFilterRequest stockOrderFilterRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockOrderEntity> stockOrderEntityList;
//        if (stockOrderFilterRequest.getStatus() == null) {
//            stockOrderFilterRequest.setStatus(0 + "");
//        }

        if (StringUtils.isNotBlank(stockOrderFilterRequest.getCorpId())) {
            CompanyEntity companyEntity = companyService.findCompany(Long.parseLong(stockOrderFilterRequest.getCorpId()));
            stockOrderFilterRequest.setUnitIdFk(companyEntity.getUnitIdFk());
            stockOrderFilterRequest.setCorpId(null);
        }

        stockOrderEntityList = stockOrderService.filterListOr(stockOrderFilterRequest);
        PageInfo<StockOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(stockOrderEntityList);
        PageSimpleResponse<StockOrderEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockOrderEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwms/stock/order/filterDetail")
    public BaseResponse filterDetail(StockOrderFilterRequest stockOrderFilterRequest,
                                     BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (stockOrderFilterRequest.getBillType() == null) {
            stockOrderFilterRequest.setBillType(stockOrderFilterRequest.getBillAction());
        }
        List<ErpOrderResponse> erpOrderResponses = new ArrayList<>();
        if (stockOrderFilterRequest.getCustomerId() != null) {
            CompanyEntity companyEntity = companyService.findCompany(Long.parseLong(stockOrderFilterRequest.getCustomerId()));
            stockOrderFilterRequest.setUnitIdFk(companyEntity.getUnitIdFk());
        }

        List<StockOrderEntity> stockOrderEntityList;
        stockOrderEntityList = stockOrderService.filterListOr(stockOrderFilterRequest);
        if (stockOrderEntityList != null && stockOrderEntityList.size() > 0) {


            for (StockOrderEntity stockOrderEntity : stockOrderEntityList) {
                ErpOrderResponse erpOrderResponse = new ErpOrderResponse();
                BeanUtils.copyProperties(stockOrderEntity, erpOrderResponse);

                StockOrderDetailFilterRequest stockOrderDetailFilterRequest = new StockOrderDetailFilterRequest();
                stockOrderDetailFilterRequest.setOrderIdFk(stockOrderEntity.getId());
                List<StockOrderDetailEntity> stockOrderDetailEntityList
                        = stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);

                List<ErpOrderResponse.SubErpOrder> subErpOrders = new ArrayList<>();
                if (stockOrderDetailEntityList != null && stockOrderDetailEntityList.size() > 0) {
                    for (StockOrderDetailEntity thrOrderDetailEntity : stockOrderDetailEntityList) {
                        ErpOrderResponse.SubErpOrder subErpOrder = new ErpOrderResponse.SubErpOrder();
                        BeanUtils.copyProperties(thrOrderDetailEntity, subErpOrder);
                        subErpOrders.add(subErpOrder);
                    }
                }
                erpOrderResponse.setSubErpOrders(subErpOrders);
                erpOrderResponses.add(erpOrderResponse);
            }
        }
        PageInfo<StockOrderEntity> pageInfo;
        pageInfo = new PageInfo<>(stockOrderEntityList);
        PageSimpleResponse<ErpOrderResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(erpOrderResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwms/stock/order/submit")
    public BaseResponse submitStockOrder(@RequestParam("id") String id) {
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        stockOrderEntity.setId(id);
        stockOrderEntity.setStatus(202 + "");//已配货
        stockOrderService.updateById(stockOrderEntity);
        return ResultVOUtils.success();
    }

    @GetMapping("/udiwms/stock/order/new/submit")
    public BaseResponse submitNewStockOrder(@RequestParam("id") String id) {
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        stockOrderEntity.setId(id);
        stockOrderEntity.setStatus(502 + "");//已提交
        stockOrderService.updateById(stockOrderEntity);
        return ResultVOUtils.success();
    }

    @GetMapping("/udiwms/stock/order/new/repealSubmit")
    public BaseResponse repealSubmitNewStockOrder(@RequestParam("id") String id) {
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        stockOrderEntity.setId(id);
        stockOrderEntity.setStatus(501 + "");//未提交
        stockOrderService.updateById(stockOrderEntity);
        return ResultVOUtils.success();
    }

    @GetMapping("/udiwms/stock/order/error/submit")
    public BaseResponse submitStockOrderError(@RequestParam("id") String id) {
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        stockOrderEntity.setId(id);
        stockOrderEntity.setStatus(101 + "");//重新提交为未处理
        stockOrderService.updateById(stockOrderEntity);
        return ResultVOUtils.success();
    }

    @PostMapping("/udiwms/stock/order/qrcode/generate/text")
    public BaseResponse generateStockOrderQRCodeText(@RequestBody StockOrderIdsRequest stockOrderIdsRequest) {

        if (stockOrderIdsRequest.getIds().size() < 1) return ResultVOUtils.error(ResultEnum.DATA_ERROR);
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        for (String id : stockOrderIdsRequest.getIds()) {
            stockOrderFilterRequest.setId(id);
            StockOrderEntity stockOrderEntity = stockOrderService.findOne(stockOrderFilterRequest);
            stockPrintService.deleteByOrderId(stockOrderEntity.getId());
            if (stockOrderEntity == null) continue;
            StockOrderDetailFilterRequest stockOrderDetailFilterRequest = new StockOrderDetailFilterRequest();
            stockOrderDetailFilterRequest.setLimit(100);
            stockOrderDetailFilterRequest.setPage(1);
            stockOrderDetailFilterRequest.setOrderIdFk(id);
            List<StockOrderDetailEntity> stockOrderDetailEntityList =
                    stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);

            List<StockPrintEntity> stockPrintEntities = new ArrayList<>();
            for (StockOrderDetailEntity stockOrderDetailEntity : stockOrderDetailEntityList) {
                if (stockOrderIdsRequest.isFilterUDIType()) {
                    UdiInfoEntity udiInfoEntity =
                            udiInfoService.findByNameCode(stockOrderDetailEntity.getProductId());
                    if (udiInfoEntity != null) {//TODO udi过滤检测
                        StockPrintEntity stockPrintEntity = new StockPrintEntity();
                        BeanUtils.copyProperties(udiInfoEntity, stockPrintEntity);
                        stockPrintEntity.setSOrderId(stockOrderEntity.getId());
                        stockPrintEntity.setUpdateTime(new Date());
                        stockPrintEntity.setProduceDate(stockOrderDetailEntity.getProductDate());
                        stockPrintEntity.setExpireDate(stockOrderDetailEntity.getExpireDate());
                        stockPrintEntity.setBatchNo(stockOrderDetailEntity.getBatchNo());
                        stockPrintEntity.setSDetailId(stockOrderDetailEntity.getId());
                        stockPrintEntities.add(stockPrintEntity);
                    }
                }

            }

            PostStockPrintRequest postStockPrintRequest = new PostStockPrintRequest();
            postStockPrintRequest.setThirdSys(stockOrderEntity.getThirdSysFk());
            postStockPrintRequest.setDatas(stockPrintEntities);
            if (stockPrintEntities.size() > 0) {
                generateQRCodeText(postStockPrintRequest);
            }
        }
        return ResultVOUtils.success("操作成功，生成中。");
    }

    private String generateQRCodeText(PostStockPrintRequest postStockPrintRequest) {
        String genKey = null;
        if (postStockPrintRequest != null) {

//            InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
//            inCodeLogEntity.setFromType(IncodeStatus.FROM_THRSYS);
//            inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
//            if (postStockPrintRequest.getGenKey() == null) {
//                genKey = CustomUtil.getId();
//                inCodeLogEntity.setGenKey(genKey);
//            } else {
//                genKey = postStockPrintRequest.getGenKey();
//                inCodeLogEntity.setGenKey(postStockPrintRequest.getGenKey());
//            }
//            inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
//            inCodeLogEntity.setThirdSysFk(postStockPrintRequest.getThirdSys());
//            inCodeLogService.insertInCodeLog(inCodeLogEntity);

            List<StockPrintEntity> stockPrintEntities = postStockPrintRequest.getDatas();
            if (stockPrintEntities != null && stockPrintEntities.size() > 0) {
                for (StockPrintEntity stockPrintEntity : stockPrintEntities) {
                    stockPrintEntity.setGenKeyFk(genKey);
                    String udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                            "#" + stockPrintEntity.getBatchNo() + "#" + stockPrintEntity.getSerialNo() + "#" + stockPrintEntity.getWarehouseCode() + "#" + stockPrintEntity.getSpaceCode();
                    stockPrintEntity.setUdiCode(udiCode);
                    stockPrintService.insertStockPrint(stockPrintEntity);

                    //--------------------------
                    StockQRCodeTextEntity stockQRCodeTextEntity = new StockQRCodeTextEntity();
                    stockQRCodeTextEntity.setOrderId(stockPrintEntity.getSOrderId());
                    stockQRCodeTextEntity.setDetailId(stockPrintEntity.getSDetailId());
                    stockQRCodeTextEntity.setStatus(101 + "");//生成完成
                    stockQRCodeTextEntity.setText(genKey);
                    Date tDate = new Date();
                    stockQRCodeTextEntity.setCreate_time(tDate);
                    stockQRCodeTextEntity.setUpdate_time(tDate);
                    stockQRCodeTextService.insertStockQRCodeText(stockQRCodeTextEntity);
                }
            }
//            inCodeLogEntity.setStatus(IncodeStatus.FINISH);
//            inCodeLogService.updateInCodeLog(inCodeLogEntity);
        }
        return genKey;
    }

    @GetMapping("/udiwms/stock/order/generateBillNo")
    public BaseResponse stockOrderGenerateBillNo(@RequestParam("action") String action) {

        String billNo = StockOrderBillNoUtils.getBillNo(action);

        return ResultVOUtils.success(billNo);
    }
}
