package com.glxp.sale.admin.controller.inout;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.sale.admin.entity.basic.UdiInfoEntity;
import com.glxp.sale.admin.entity.inout.StockOrderDetailEntity;
import com.glxp.sale.admin.entity.inout.OrderEntity;
import com.glxp.sale.admin.entity.inout.StockQRCodeTextEntity;
import com.glxp.sale.admin.entity.inout.WarehouseEntity;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.entity.udid.UdiEntity;
import com.glxp.sale.admin.httpclient.ErpOrderClient;
import com.glxp.sale.admin.req.basic.CompanyProductRelevanceRequest;
import com.glxp.sale.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.sale.admin.req.inout.*;
import com.glxp.sale.admin.req.param.SystemParamConfigRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.basic.CompanyProductRelevanceResponse;
import com.glxp.sale.admin.res.basic.UdiRelevanceResponse;
import com.glxp.sale.admin.res.inout.PurchaseinResponse;
import com.glxp.sale.admin.res.inout.StockOrderDetailQueryProductResponse;
import com.glxp.sale.admin.res.inout.StockOrderDetailResponse;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.info.CompanyProductRelevanceService;
import com.glxp.sale.admin.service.inout.*;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.thread.InvDlErpStockService;
import com.glxp.sale.admin.thread.IoTransInoutService;
import com.glxp.sale.admin.util.FilterUdiUtils;
import com.glxp.sale.admin.util.HttpClient;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@RestController
public class StockOrderDetailController {

    @Resource
    StockOrderDetailService stockOrderDetailService;
    @Resource
    StockOrderDetailTempService stockOrderDetailTempService;
    @Resource
    StockQRCodeTextService stockQRCodeTextService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private CompanyProductRelevanceService companyProductRelevanceService;

    @Resource
    private IoTransInoutService transInoutService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;

    @GetMapping("/udiwms/stock/order/detail/filter")
    public BaseResponse filterStockOrderDetail(StockOrderDetailFilterRequest stockOrderDetailFilterRequest,
                                               BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockOrderDetailEntity> stockOrderDetailEntityList
                = stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);
        PageInfo<StockOrderDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(stockOrderDetailEntityList);
        PageSimpleResponse<StockOrderDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockOrderDetailEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/stock/order/detail/filter2")
    public BaseResponse filterStockOrderDetail2(StockOrderDetailFilterRequest stockOrderDetailFilterRequest,
                                                BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockOrderDetailResponse> stockOrderDetailResponseList
                = stockOrderDetailService.filterStockOrderDetail2(stockOrderDetailFilterRequest);
        PageInfo<StockOrderDetailResponse> pageInfo;
        for (StockOrderDetailResponse stockOrderDetailResponse : stockOrderDetailResponseList) {
            stockOrderDetailResponse.setProductName(stockOrderDetailResponse.getCpmctymc());
            stockOrderDetailResponse.setSpec(stockOrderDetailResponse.getGgxh());
        }
        pageInfo = new PageInfo<>(stockOrderDetailResponseList);
        PageSimpleResponse<StockOrderDetailResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockOrderDetailResponseList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/stock/order/detail/temp/filter")
    public BaseResponse filterStockOrderDetailTemp(StockOrderDetailFilterRequest stockOrderDetailFilterRequest,
                                                   BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockOrderDetailEntity> stockOrderDetailEntityList
                = stockOrderDetailTempService.filterStockOrderDetailTemp(stockOrderDetailFilterRequest);
        PageInfo<StockOrderDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(stockOrderDetailEntityList);
        PageSimpleResponse<StockOrderDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockOrderDetailEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("/udiwms/stock/order/detail/getStockOrderDetails")
    public BaseResponse getStockOrderDetails(StockOrderDetailFilterRequest stockOrderDetailFilterRequest,
                                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockOrderDetailEntity> stockOrderDetailEntityList
                = stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);
        PageInfo<StockOrderDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(stockOrderDetailEntityList);
        PageSimpleResponse<StockOrderDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockOrderDetailEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping(value = "/udiwms/stock/order/detail")
    public BaseResponse stockOrderDetail(StockOrderDetailFilterRequest stockOrderDetailFilterRequest,
                                         BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (stockOrderDetailFilterRequest.getOrderIdFk() == null || stockOrderDetailFilterRequest.getOrderIdFk().equals("")) {
            stockOrderDetailFilterRequest.setOrderIdFk("1");
        }
        stockOrderDetailFilterRequest.setPage(1);
        stockOrderDetailFilterRequest.setLimit(100);
        List<StockOrderDetailEntity> erpOrderEntities = stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);
        return ResultVOUtils.success(erpOrderEntities);
    }

    @GetMapping("/udiwms/stock/order/detail/uploadCount")
    public BaseResponse uploadCount(@RequestParam("orderId") String orderId, @RequestParam("code") String code) {
        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null)
            return  ResultVOUtils.error(ResultEnum.DATA_ERROR, "条码错误 1");
        if (StringUtils.isBlank(udiEntity.getUdi()))
            return ResultVOUtils.error(ResultEnum.DATA_ERROR, "条码错误 2");

        UdiInfoEntity udiInfoEntity =
                udiInfoService.findByNameCode(udiEntity.getUdi());
        if (udiInfoEntity == null) {
            return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到该产品");
        }

        StockOrderDetailFilterRequest stockOrderDetailFilterRequest = new StockOrderDetailFilterRequest();
        stockOrderDetailFilterRequest.setOrderIdFk(orderId);
        stockOrderDetailFilterRequest.setProductId(udiEntity.getUdi());
        List<StockOrderDetailEntity> stockOrderDetailEntityList =
                stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);
        if (stockOrderDetailEntityList.size() > 0) {
            SystemParamConfigRequest systemParamConfigRequest = new SystemParamConfigRequest();
            systemParamConfigRequest.setParamKey("stock_order_detail_count_max");
            SystemParamConfigEntity systemParamConfigEntitySODCountMax =
                    systemParamConfigService.selectByParamKey(systemParamConfigRequest);

            StockOrderDetailEntity tempStockOrderDetailEntity = stockOrderDetailEntityList.get(0);
            StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
            stockOrderDetailEntity.setId(tempStockOrderDetailEntity.getId());

            if (systemParamConfigEntitySODCountMax == null ||
                    StringUtils.isNotBlank(systemParamConfigEntitySODCountMax.getParamValue()) &&
                            systemParamConfigEntitySODCountMax.getParamValue().equals("1")) {
                if (Integer.parseInt(tempStockOrderDetailEntity.getCount()) <
                        Integer.parseInt(tempStockOrderDetailEntity.getReCount())) {
                    return ResultVOUtils.error(500, "配货数量超过订单数量");
                }
            }
            stockOrderDetailEntity.setReCount(
                    String.valueOf(Integer.parseInt(tempStockOrderDetailEntity.getReCount()) + 1));
            stockOrderDetailService.updateById(stockOrderDetailEntity);

            return ResultVOUtils.success(orderId);
        }

        return ResultVOUtils.error(ResultEnum.DATA_NOT);
    }

    @GetMapping("/udiwms/stock/order/detail/getInstrument")
    public BaseResponse getStockOrderDetailInstrument(@RequestParam("count") String count, @RequestParam("code") String code) {
        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null) ResultVOUtils.error(ResultEnum.DATA_ERROR, "条码错误 1");

        UdiInfoEntity udiInfoEntity =
                udiInfoService.findByNameCode(udiEntity.getUdi());
        if (udiInfoEntity != null) {
            StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
            stockOrderDetailEntity.setProductId(udiInfoEntity.getNameCode());
            stockOrderDetailEntity.setProductName(udiInfoEntity.getCpmctymc());
            stockOrderDetailEntity.setSpec(udiInfoEntity.getGgxh());
            stockOrderDetailEntity.setBatchNo(udiEntity.getBatchNo());
            stockOrderDetailEntity.setProductDate(udiEntity.getProduceDate());
            stockOrderDetailEntity.setExpireDate(udiEntity.getExpireDate());
            stockOrderDetailEntity.setCount(count);

            return ResultVOUtils.success(stockOrderDetailEntity);
        }

        return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到该产品");
    }

    @PostMapping("/udiwms/stock/order/detail/getInstrumentById")
    public BaseResponse getStockOrderDetailInstrumentById(@RequestBody StockOrderIdsRequest stockOrderIdsRequest) {

        List<StockOrderDetailEntity> stockOrderDetailEntityList = new ArrayList<>();
        for (String id : stockOrderIdsRequest.getIds()) {
            CompanyProductRelevanceRequest companyProductRelevanceRequest = new CompanyProductRelevanceRequest();
            companyProductRelevanceRequest.setId(id);
            List<CompanyProductRelevanceResponse> companyProductRelevanceResponses = companyProductRelevanceService.filterUdiGp(companyProductRelevanceRequest);

            if (companyProductRelevanceResponses != null && companyProductRelevanceResponses.size() > 0) {
                CompanyProductRelevanceResponse udiInfoEntity = companyProductRelevanceResponses.get(0);
                StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
                stockOrderDetailEntity.setProductId(udiInfoEntity.getThirdId());
                stockOrderDetailEntity.setProductName(udiInfoEntity.getCpmctymc());
                stockOrderDetailEntity.setSpec(udiInfoEntity.getGgxh());
                if (stockOrderIdsRequest.getUdiEntity() != null) {
                    UdiEntity udiEntity = stockOrderIdsRequest.getUdiEntity();
                    stockOrderDetailEntity.setBatchNo(udiEntity.getBatchNo());
                    stockOrderDetailEntity.setProductDate(udiEntity.getProduceDate());
                    stockOrderDetailEntity.setExpireDate(udiEntity.getExpireDate());
                }
                stockOrderDetailEntity.setCount(0 + "");
                stockOrderDetailEntity.setReCount(0 + "");
                stockOrderDetailEntity.setYlqxzcrbarmc(udiInfoEntity.getYlqxzcrbarmc());
                stockOrderDetailEntity.setZczbhhzbapzbh(udiInfoEntity.getZczbhhzbapzbh());
                stockOrderDetailEntityList.add(stockOrderDetailEntity);
            }
        }
        if (stockOrderDetailEntityList.size() == 0) {
            return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到产品");
        }
        return ResultVOUtils.success(stockOrderDetailEntityList);
    }

    @GetMapping("/udiwms/stock/order/detail/addFromCode")
    public BaseResponse addStockOrderDetailFromCode(@RequestParam("orderId") String orderId,
                                                    @RequestParam("count") String count,
                                                    @RequestParam("code") String code) {
        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null) ResultVOUtils.error(ResultEnum.DATA_ERROR, "条码错误 1");

        UdiInfoEntity udiInfoEntity =
                udiInfoService.findByNameCode(udiEntity.getUdi());
        if (udiInfoEntity != null) {
            StockOrderDetailFilterRequest stockOrderDetailFilterRequest = new StockOrderDetailFilterRequest();
            stockOrderDetailFilterRequest.setOrderIdFk(orderId);
            stockOrderDetailFilterRequest.setProductId(udiEntity.getUdi());
            stockOrderDetailFilterRequest.setBatchNo(udiEntity.getBatchNo());
            stockOrderDetailFilterRequest.setProductDate(udiEntity.getProduceDate());
            stockOrderDetailFilterRequest.setExpireDate(udiEntity.getExpireDate());
            if (stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest).size() > 0) {
                return ResultVOUtils.error(ResultEnum.DATA_REPEAT, "已添加");
            }

            StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
            stockOrderDetailEntity.setOrderIdFk(orderId);
            stockOrderDetailEntity.setProductId(udiInfoEntity.getNameCode());
            stockOrderDetailEntity.setProductName(udiInfoEntity.getCpmctymc());
            stockOrderDetailEntity.setSpec(udiInfoEntity.getGgxh());
            stockOrderDetailEntity.setBatchNo(udiEntity.getBatchNo());
            stockOrderDetailEntity.setProductDate(udiEntity.getProduceDate());
            stockOrderDetailEntity.setExpireDate(udiEntity.getExpireDate());
            stockOrderDetailEntity.setCount(count);
            stockOrderDetailEntity.setReCount(count);
            stockOrderDetailEntity.setSweepCount(0 + "");
            stockOrderDetailEntity.setYlqxzcrbarmc(udiInfoEntity.getYlqxzcrbarmc());
            stockOrderDetailEntity.setZczbhhzbapzbh(udiInfoEntity.getZczbhhzbapzbh());

            stockOrderDetailService.insertStockOrderDetail(stockOrderDetailEntity);

            return ResultVOUtils.success();
        }

        return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到该产品");
    }

    @GetMapping("/udiwms/stock/order/detail/addFromCodeById")
    public BaseResponse addStockOrderDetailFromCodeById(@RequestParam("orderId") String orderId,
                                                        @RequestParam("id") String id) {
        UdiInfoEntity udiInfoEntity =
                udiInfoService.findById(id);
        if (udiInfoEntity != null) {

            StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
            stockOrderDetailEntity.setOrderIdFk(orderId);
            stockOrderDetailEntity.setProductId(udiInfoEntity.getNameCode());
            stockOrderDetailEntity.setProductName(udiInfoEntity.getCpmctymc());
            stockOrderDetailEntity.setSpec(udiInfoEntity.getGgxh());
            stockOrderDetailEntity.setBatchNo(udiInfoEntity.getBatchNo());
            stockOrderDetailEntity.setProductDate(udiInfoEntity.getProduceDate());
            stockOrderDetailEntity.setExpireDate(udiInfoEntity.getExpireDate());
            stockOrderDetailEntity.setCount(0 + "");
            stockOrderDetailEntity.setReCount(0 + "");
            stockOrderDetailEntity.setSweepCount(0 + "");
            stockOrderDetailEntity.setYlqxzcrbarmc(udiInfoEntity.getYlqxzcrbarmc());
            stockOrderDetailEntity.setZczbhhzbapzbh(udiInfoEntity.getZczbhhzbapzbh());

            stockOrderDetailService.insertStockOrderDetail(stockOrderDetailEntity);

            return ResultVOUtils.success();
        }

        return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到该产品");
    }

    @GetMapping("/udiwms/stock/order/detail/queryProduct")
    public BaseResponse queryProduct(@RequestParam("code") String code, @RequestParam("customerId") String customerId) {
        UdiEntity udiEntity = null;
        if (StringUtils.isNotBlank(code)) {
            CompanyProductRelevanceRequest companyProductRelevanceRequest = new CompanyProductRelevanceRequest();
            companyProductRelevanceRequest.setCustomerId(customerId);

            if (code.length() > 13) {
                String tStr = code.substring(0, 2);
                if (tStr.contains("MA") || tStr.contains("#") || tStr.contains("01")) {
                    try {
                        udiEntity = FilterUdiUtils.getUdi(code);
                    } catch (Exception e) {

                    }
                }
            }
            StockOrderDetailQueryProductResponse stockOrderDetailQueryProductResponse = new StockOrderDetailQueryProductResponse();
            if (udiEntity == null) {
                companyProductRelevanceRequest.setUnionCode(code);
                stockOrderDetailQueryProductResponse.setUdi(code);
            } else {
                companyProductRelevanceRequest.setUnionCode(udiEntity.getUdi());
                BeanUtils.copyProperties(udiEntity, stockOrderDetailQueryProductResponse);
            }
            List<CompanyProductRelevanceResponse> basicInstrumentMaintainRelevanceResponses = null;
            basicInstrumentMaintainRelevanceResponses = companyProductRelevanceService.filterUdiGp(companyProductRelevanceRequest);
            if (basicInstrumentMaintainRelevanceResponses.size() == 1) {
                stockOrderDetailQueryProductResponse.setGetType("1");
                stockOrderDetailQueryProductResponse.setGetId(basicInstrumentMaintainRelevanceResponses.get(0).getId() + "");
            } else if (basicInstrumentMaintainRelevanceResponses.size() > 1) {
                stockOrderDetailQueryProductResponse.setGetType("2");
            } else {
                return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到产品");
            }
            return ResultVOUtils.success(stockOrderDetailQueryProductResponse);
        } else {
            return ResultVOUtils.error(ResultEnum.DATA_NOT, "条码不能为空");
        }

    }

    @GetMapping("/udiwms/stock/order/detail/filterProduct")
    public BaseResponse filterProduct(FilterUdiInfoRequest filterUdiInfoRequest) {

        String nameCode = null;

        UdiEntity udiEntity = null;

        if (filterUdiInfoRequest.getUdiCode() != null && !filterUdiInfoRequest.getUdiCode().isEmpty()) {
            udiEntity = FilterUdiUtils.getUdi(filterUdiInfoRequest.getUdiCode());
        }

        if (udiEntity != null) {
            nameCode = udiEntity.getUdi();
        } else {
            nameCode = filterUdiInfoRequest.getNameCode();
        }

        filterUdiInfoRequest.setNameCode(nameCode);
        CompanyProductRelevanceRequest companyProductRelevanceRequest = new CompanyProductRelevanceRequest();
        BeanUtils.copyProperties(filterUdiInfoRequest, companyProductRelevanceRequest);

        List<CompanyProductRelevanceResponse> basicInstrumentMaintainRelevanceResponses = null;
        basicInstrumentMaintainRelevanceResponses = companyProductRelevanceService.filterUdiGp(companyProductRelevanceRequest);
        if (basicInstrumentMaintainRelevanceResponses != null && basicInstrumentMaintainRelevanceResponses.size() > 0) {
            for (CompanyProductRelevanceResponse basicInstrumentMaintainRelevanceResponse : basicInstrumentMaintainRelevanceResponses) {
                if (basicInstrumentMaintainRelevanceResponse.getDiType() == 2 && basicInstrumentMaintainRelevanceResponse.getIsUseDy() != 1) {
                    basicInstrumentMaintainRelevanceResponse.setNameCode(basicInstrumentMaintainRelevanceResponse.getSjcpbm());
                }
            }
        }

        PageInfo<CompanyProductRelevanceResponse> pageInfo;
        pageInfo = new PageInfo<>(basicInstrumentMaintainRelevanceResponses);
        PageSimpleResponse<CompanyProductRelevanceResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(basicInstrumentMaintainRelevanceResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/udiwms/stock/order/detail/copy")
    public BaseResponse uploadCount(@RequestParam("id") String id) {
        if (StringUtils.isNotBlank(id)) {
            StockOrderDetailEntity stockOrderDetailEntity = stockOrderDetailService.findOne(id);
            if (stockOrderDetailEntity != null) {
                stockOrderDetailEntity.setId(null);
//                stockOrderDetailEntity.setReCount(0 + "");
                stockOrderDetailService.insertStockOrderDetail(stockOrderDetailEntity);
                return ResultVOUtils.success();
            }
        }
        return ResultVOUtils.error(ResultEnum.DATA_NOT);
    }


    @GetMapping(value = "/udiwms/stock/order/detail/upload")
    public BaseResponse upload(StockOrderDetailFilterRequest stockOrderDetailFilterRequest,
                               BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (StringUtils.isNotBlank(stockOrderDetailFilterRequest.getId())) {
            if (StringUtils.isBlank(stockOrderDetailFilterRequest.getCount()) ||
                    StringUtils.isBlank(stockOrderDetailFilterRequest.getReCount())) {
                return ResultVOUtils.error(500, "获取数量失败");
            }
            SystemParamConfigRequest systemParamConfigRequest = new SystemParamConfigRequest();
            systemParamConfigRequest.setParamKey("stock_order_detail_count_max");
            SystemParamConfigEntity systemParamConfigEntitySODCountMax = systemParamConfigService.selectByParamKey(systemParamConfigRequest);
            if (systemParamConfigEntitySODCountMax == null) return ResultVOUtils.error(500, "获取设置失败");

            if (StringUtils.isNotBlank(systemParamConfigEntitySODCountMax.getParamValue()) &&
                    systemParamConfigEntitySODCountMax.getParamValue().equals("1")) {
                if (Integer.parseInt(stockOrderDetailFilterRequest.getCount()) <
                        Integer.parseInt(stockOrderDetailFilterRequest.getReCount())) {
                    return ResultVOUtils.error(500, "数量超过订单数量");
                }
            }

            StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
            BeanUtils.copyProperties(stockOrderDetailFilterRequest, stockOrderDetailEntity);
            boolean b = stockOrderDetailService.updateById(stockOrderDetailEntity);
            if (b) {
                return ResultVOUtils.success();
            }
        }
        return ResultVOUtils.error(500, "修改失败");
    }

    @PostMapping("/udiwms/stock/order/detail/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = stockOrderDetailService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }

    @PostMapping("/udiwms/stock/order/detail/temp/delete")
    public BaseResponse deleteTemp(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = stockOrderDetailTempService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }
}
