package com.glxp.sale.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.entity.basic.BussinessTypeEntity;
import com.glxp.sale.admin.entity.inout.*;
import com.glxp.sale.admin.entity.param.SystemPDFModuleEntity;
import com.glxp.sale.admin.entity.param.SystemPDFTemplateEntity;
import com.glxp.sale.admin.req.inout.DeleteRequest;
import com.glxp.sale.admin.req.inout.StockOrderDetailFilterRequest;
import com.glxp.sale.admin.req.inout.StockQRCodeTextFilterRequest;
import com.glxp.sale.admin.req.itextpdf.InspectionPDFTemplateRequest;
import com.glxp.sale.admin.req.itextpdf.StockQRCodeTextPDFTemplateRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.basic.BussinessTypeService;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.inout.*;
import com.glxp.sale.admin.service.param.SystemPDFModuleService;
import com.glxp.sale.admin.service.param.SystemPDFTemplateService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.thread.IoTransInoutService;
import com.glxp.sale.admin.util.ITextPDFUtils;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

@RestController
public class StockQRCodeTextController {

    @Resource
    StockOrderService stockOrderService;
    @Resource
    StockOrderDetailService stockOrderDetailService;
    @Resource
    StockOrderDetailTempService stockOrderDetailTempService;
    @Resource
    StockQRCodeTextService stockQRCodeTextService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    private UdiRelevanceService udiRelevanceService;
    @Resource
    private SystemPDFTemplateService systemPDFTemplateService;
    @Resource
    private SystemPDFModuleService systemPDFModuleService;
    @Resource
    private BussinessTypeService bussinessTypeService;


    @Resource
    private IoTransInoutService transInoutService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;

    @GetMapping("/udiwms/stock/qrcode/text/filter")
    public BaseResponse filterStockQRCodeText(StockQRCodeTextFilterRequest stockQRCodeTextFilterRequest,
                                              BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<StockQRCodeTextEntity> stockQRCodeTextEntityList
                = stockQRCodeTextService.filterStockQRCodeText(stockQRCodeTextFilterRequest);
        for (StockQRCodeTextEntity stockQRCodeTextEntity : stockQRCodeTextEntityList) {
            StockOrderDetailEntity stockOrderDetailEntity = stockOrderDetailService.findOne(stockQRCodeTextEntity.getDetailId());
            if (stockOrderDetailEntity != null) {
                stockQRCodeTextEntity.setCount(stockOrderDetailEntity.getCount());
                stockQRCodeTextEntity.setReCount(stockOrderDetailEntity.getReCount());
            } else {
                stockQRCodeTextEntity.setCount("0");
                stockQRCodeTextEntity.setReCount("0");
            }
        }
        PageInfo<StockQRCodeTextEntity> pageInfo;
        pageInfo = new PageInfo<>(stockQRCodeTextEntityList);
        PageSimpleResponse<StockQRCodeTextEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockQRCodeTextEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    //-----------打印码
    @PostMapping("/udiwms/pdf/template/inspection/stock/qrcode/text/file")
    public BaseResponse inspectionStockQRCodeTextPDFFromTemplateFile(
            @RequestBody InspectionPDFTemplateRequest inspectionPDFTemplateRequest) throws Exception {

        StockQRCodeTextEntity stockQRCodeTextEntity =
                stockQRCodeTextService.findOne(inspectionPDFTemplateRequest.getQueryId());
        if (stockQRCodeTextEntity == null) {
            return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到该码");
        }

        SystemPDFModuleEntity systemPDFModuleEntity = systemPDFModuleService.selectById(3 + "");
        if (systemPDFModuleEntity == null) return ResultVOUtils.error(ResultEnum.DATA_NOT, "所属模块错误");
        SystemPDFTemplateEntity systemPDFTemplateEntity =
                systemPDFTemplateService.selectById(String.valueOf(systemPDFModuleEntity.getTemplateId()));
        if (systemPDFTemplateEntity == null) return ResultVOUtils.error(ResultEnum.DATA_NOT, "模板错误");

        return ResultVOUtils.success();
    }
    @PostMapping("/udiwms/pdf/template/stock/qrcode/text/file")
    public void stockQRCodeTextPDFFromTemplateFile(
            @RequestBody StockQRCodeTextPDFTemplateRequest stockQRCodeTextPDFTemplateRequest,
            HttpServletRequest request, HttpServletResponse response) throws Exception {

        StockQRCodeTextEntity stockQRCodeTextEntity = stockQRCodeTextService.findOne(stockQRCodeTextPDFTemplateRequest.getId());
        if (stockQRCodeTextEntity == null) {
            return;
        }

        SystemPDFModuleEntity systemPDFModuleEntity = systemPDFModuleService.selectById(3 + "");
        if (systemPDFModuleEntity == null) return;
        SystemPDFTemplateEntity systemPDFTemplateEntity =
                systemPDFTemplateService.selectById(String.valueOf(systemPDFModuleEntity.getTemplateId()));
        if (systemPDFTemplateEntity == null) return;
        stockQRCodeTextPDFTemplateRequest.setPath(systemPDFTemplateEntity.getPath());
        //-----------------------------------------------
        stockQRCodeTextPDFTemplateRequest.setOrderId(stockQRCodeTextEntity.getOrderId());
        stockQRCodeTextPDFTemplateRequest.setDetailId(stockQRCodeTextEntity.getDetailId());

        ITextPDFUtils.exportStockQRCodeTextPDFFromTemplateFile(
                stockQRCodeTextPDFTemplateRequest, request, response);

        StockOrderEntity updateStockOrderEntity = new StockOrderEntity();
        updateStockOrderEntity.setId(stockQRCodeTextEntity.getOrderId());
        updateStockOrderEntity.setPrintStatus("102");
        stockOrderService.updateById(updateStockOrderEntity);
    }

    //-----------打印订单
    @PostMapping("/udiwms/pdf/template/inspection/stock/order/file")
    public BaseResponse inspectionStockOrderPDFFromTemplateFile(
            @RequestBody InspectionPDFTemplateRequest inspectionPDFTemplateRequest) throws Exception {

        StockOrderEntity stockOrderEntity =
                stockOrderService.findById(inspectionPDFTemplateRequest.getOrderId());
        if (stockOrderEntity == null) {
            return ResultVOUtils.error(ResultEnum.DATA_NOT, "未找到该订单");
        }

        SystemPDFModuleEntity systemPDFModuleEntity = systemPDFModuleService.selectById(2 + "");
        if (systemPDFModuleEntity == null) return ResultVOUtils.error(ResultEnum.DATA_NOT, "所属模块错误");
        SystemPDFTemplateEntity systemPDFTemplateEntity =
                systemPDFTemplateService.selectById(String.valueOf(systemPDFModuleEntity.getTemplateId()));
        if (systemPDFTemplateEntity == null) return ResultVOUtils.error(ResultEnum.DATA_NOT, "模板错误");

        return ResultVOUtils.success();
    }
    @PostMapping("/udiwms/pdf/template/stock/order/file")
    public void stockOrderPDFFromTemplateFile(
            @RequestBody StockQRCodeTextPDFTemplateRequest stockQRCodeTextPDFTemplateRequest,
            HttpServletRequest request, HttpServletResponse response) throws Exception {

        StockOrderEntity stockOrderEntity =
                stockOrderService.findById(stockQRCodeTextPDFTemplateRequest.getId());
        if (stockOrderEntity == null) {
            return;
        }

        SystemPDFModuleEntity systemPDFModuleEntity = systemPDFModuleService.selectById(2 + "");
        if (systemPDFModuleEntity == null) return;
        SystemPDFTemplateEntity systemPDFTemplateEntity =
                systemPDFTemplateService.selectById(String.valueOf(systemPDFModuleEntity.getTemplateId()));
        if (systemPDFTemplateEntity == null) return;
        stockQRCodeTextPDFTemplateRequest.setPath(systemPDFTemplateEntity.getPath());
        stockQRCodeTextPDFTemplateRequest.setRowCount(systemPDFTemplateEntity.getRowCount());
        stockQRCodeTextPDFTemplateRequest.setQrcodeCount(systemPDFTemplateEntity.getQrcodeCount());
        //-----------------------------------------------
        stockQRCodeTextPDFTemplateRequest.setOrderId(stockOrderEntity.getId());

        stockQRCodeTextPDFTemplateRequest.setBillNo(stockOrderEntity.getBillNo());

        BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(stockOrderEntity.getBillType());
        stockQRCodeTextPDFTemplateRequest.setActionName(bussinessTypeEntity == null ? " " : bussinessTypeEntity.getName());

        StockOrderDetailFilterRequest stockOrderDetailFilterRequest = new StockOrderDetailFilterRequest();
        stockOrderDetailFilterRequest.setOrderIdFk(stockOrderEntity.getId());
        List<StockOrderDetailEntity> stockOrderDetailEntityList = stockOrderDetailService.filterStockOrderDetail(stockOrderDetailFilterRequest);

        ITextPDFUtils.exportStockOrderPDFFromTemplateFile(
                stockQRCodeTextPDFTemplateRequest, stockOrderEntity, stockOrderDetailEntityList, request, response);

        StockOrderEntity updateStockOrderEntity = new StockOrderEntity();
        updateStockOrderEntity.setId(stockOrderEntity.getId());
        updateStockOrderEntity.setPrintStatus("102");
        stockOrderService.updateById(updateStockOrderEntity);
    }


    @GetMapping(value = "/udiwms/stock/qrcode/text/upload")
    public BaseResponse upload(StockQRCodeTextFilterRequest stockQRCodeTextFilterRequest,
                               BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        StockQRCodeTextEntity stockQRCodeTextEntity = new StockQRCodeTextEntity();
        BeanUtils.copyProperties(stockQRCodeTextFilterRequest, stockQRCodeTextEntity);
        boolean b = stockQRCodeTextService.updateById(stockQRCodeTextEntity);
        if (b) {
            return ResultVOUtils.success();
        }
        return ResultVOUtils.error(500, "修改失败");
    }

    @PostMapping("/udiwms/stock/qrcode/text/delete")
    public BaseResponse delete(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        boolean b = stockQRCodeTextService.deleteById(deleteRequest.getId());
        if (b) {
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败");
        }
    }
}
