package com.glxp.sale.admin.controller.inout;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.sale.admin.controller.inout.utils.InoutUtils;
import com.glxp.sale.admin.entity.basic.BussinessTypeEntity;
import com.glxp.sale.admin.entity.info.CompanyEntity;
import com.glxp.sale.admin.entity.inout.ErpOrderEntity;
import com.glxp.sale.admin.entity.inout.OrderEntity;
import com.glxp.sale.admin.entity.inout.WarehouseEntity;
import com.glxp.sale.admin.entity.inout.Warehousetemp;
import com.glxp.sale.admin.entity.receipt.MipsBarcodeEntity;
import com.glxp.sale.admin.entity.receipt.ProductOrderDetailEntity;
import com.glxp.sale.admin.entity.udid.UdiEntity;
import com.glxp.sale.admin.req.inout.*;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.res.inout.WarehouseResponse;
import com.glxp.sale.admin.service.basic.BussinessTypeService;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.info.CompanyService;
import com.glxp.sale.admin.service.inout.CodesTempService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.inout.MyErpOrderService;
import com.glxp.sale.admin.service.inout.OrderService;
import com.glxp.sale.admin.service.inout.CodesService;
import com.glxp.sale.admin.service.receipt.MipsBarcodeService;
import com.glxp.sale.admin.service.receipt.ProductInfoService;
import com.glxp.sale.admin.thread.IoTransInoutService;
import com.glxp.sale.admin.util.DateUtil;
import com.glxp.sale.admin.util.FastJsonUtils;
import com.glxp.sale.admin.util.FilterUdiUtils;
import com.glxp.sale.admin.util.WarehousBeanUtils;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

@RestController
public class WareHouseController {
    @Resource
    private CodesService codesService;
    @Resource
    private OrderService orderService;
    @Resource
    private MyErpOrderService myErpOrderService;

    @Resource
    private IoTransInoutService ioTransInoutService;
    @Resource
    private CodesTempService codesTempService;
    @Resource
    CompanyService companyService;
    @Resource
    private BussinessTypeService bussinessTypeService;

    @GetMapping("admin/warehouse/inout/list")
    public BaseResponse list(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = codesService.listWarehouse(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/inout/code/deleteCodes")
    public BaseResponse deleteCodes(@RequestBody DeleteCodeRequest deleteCodeRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Long> codes = deleteCodeRequest.getCodes();
        if (codes != null && codes.size() > 0) {
            codesTempService.deleteCodesTemp(codes);
        } else {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR);
        }
        return ResultVOUtils.success("删除成功");
    }


    @GetMapping("admin/warehouse/inout/findByNo")
    public BaseResponse findByNo(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = codesService.findByNo(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("admin/warehouse/inout/findByOrderId")
    public BaseResponse findByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = codesService.findByOrderId(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("admin/warehouse/inout/findAllByOrderId")
    public BaseResponse findAllByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehouseEntityList = codesService.findAllByOrderId(warehouseQueryRequest);
        List<WarehouseResponse> warehouseResponses = warehouseEntityList.stream().map(item ->
        {
            WarehouseResponse warehouseResponse = new WarehouseResponse();
            BeanUtils.copyProperties(item, warehouseResponse);
            return warehouseResponse;
        }).collect(Collectors.toList());
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehouseEntityList);
        PageSimpleResponse<WarehouseResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehouseResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("admin/warehouse/inout/findErrorByOrderId")
    public BaseResponse findErrorByOrderId(WarehouseQueryRequest warehouseQueryRequest) {
        List<WarehouseEntity> warehousetempList = codesTempService.findByOrderId(warehouseQueryRequest);
        PageInfo<WarehouseEntity> pageInfo;
        pageInfo = new PageInfo<>(warehousetempList);
        PageSimpleResponse<WarehouseEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(warehousetempList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("/warehouse/inout/commit")//TODO 重新提交
    public BaseResponse commitOrder(@RequestBody CommitRequest commitRequest,
                                    BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        ioTransInoutService.repeatCommit(commitRequest);
        return ResultVOUtils.success("提交成功");
    }


    @GetMapping("warehouse/inout/submitOrderWeb")
    public BaseResponse submitOrderWeb(@RequestParam("orderId") String orderId) {
        orderService.updateOrderStatus(orderId, ConstantStatus.ORDER_STATUS_PROCESS);
        return ResultVOUtils.success();
    }

    @GetMapping("warehouse/inout/deleteCodesTempById")
    public BaseResponse deleteCodesTempById(@RequestParam("id") Long id) {
        WarehouseEntity warehouseEntity = new WarehouseEntity();
        warehouseEntity.setId(id);
        codesTempService.deleteCodesTempById(warehouseEntity);
        return ResultVOUtils.success();
    }

    //网页端数据上传调用接口
    @PostMapping("warehouse/inout/addOrderWeb")
    public BaseResponse addOrderWeb(@RequestBody AddOrderRequest addOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            String code = addOrderRequest.getCode();
            if (StringUtils.isBlank(code)) return ResultVOUtils.error(ResultEnum.DATA_ERROR);
            UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
            if (udiEntity == null)
                return ResultVOUtils.error(500, "UDI码格式错误！");
            String orderId = addOrderRequest.getOrderId();

            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(addOrderRequest.getBillType());
            if (StringUtils.isBlank(orderId)) {
                orderId = System.currentTimeMillis() + new Random().nextInt(10) + "";
                OrderEntity orderSaveRequest = new OrderEntity();
                orderSaveRequest.setId(orderId);
                if (addOrderRequest.getActDate() != null) {
                    orderSaveRequest.setActDate(addOrderRequest.getActDate());
                }

                CompanyEntity companyEntity = companyService.findCompany(Long.parseLong(addOrderRequest.getCustomerId()));
                orderSaveRequest.setFromCorpId(companyEntity.getUnitIdFk());
                orderSaveRequest.setFromCorp(companyEntity.getCompanyName());

                orderSaveRequest.setMainAction(bussinessTypeEntity.getMainAction());
                orderSaveRequest.setAction(bussinessTypeEntity.getAction());
                orderSaveRequest.setActor(addOrderRequest.getActor());
                orderSaveRequest.setCorpOrderId(addOrderRequest.getCorpOrderId());
//                orderSaveRequest.setFromCorpId(addOrderRequest.getFromCorpId());
//                orderSaveRequest.setFromCorp(addOrderRequest.getFromCorp());
                orderSaveRequest.setFromType(addOrderRequest.getFromType());
                orderSaveRequest.setStatus(ConstantStatus.ORDER_STATUS_TEMP_SAVE);
                orderSaveRequest.setExportStatus(ConstantStatus.ORDER_EXPORT_UN);
                orderSaveRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                orderSaveRequest.setSignStatus(ConstantStatus.ORDER_SIGN_UN);
                orderSaveRequest.setCustomerId(addOrderRequest.getCustomerId());
                orderService.insertOrder(orderSaveRequest);
            }
            WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
            warehouseQueryRequest.setOrderId(orderId);
            warehouseQueryRequest.setCode(code);
            List<WarehouseEntity> warehouseEntitys = codesTempService.findByOrderIdAndCode(warehouseQueryRequest);
            WarehouseEntity warehouseEntity = new WarehouseEntity();
            if (warehouseEntitys.size() > 0) {
                warehouseEntity = warehouseEntitys.get(0);
                warehouseEntity.setCount(warehouseEntity.getCount() + 1);
                if (StringUtils.isBlank(warehouseEntity.getSerialNo())) {
                    codesTempService.updateById(warehouseEntity);
                } else {
                    return ResultVOUtils.error(ResultEnum.DATA_REPEAT);
                }
            } else {
                warehouseEntity.setCode(code);
                warehouseEntity.setOrderId(orderId);
                warehouseEntity.setCorpOrderId(addOrderRequest.getCorpOrderId());
                warehouseEntity.setActDate(addOrderRequest.getActDate());
                warehouseEntity.setMainAction(bussinessTypeEntity.getMainAction());
                warehouseEntity.setAction(bussinessTypeEntity.getAction());
                warehouseEntity.setActor(addOrderRequest.getActor());
                warehouseEntity.setFromCorpId(addOrderRequest.getFromCorpId());
                warehouseEntity.setFromCorp(addOrderRequest.getFromCorp());
                warehouseEntity.setBatchNo(udiEntity.getBatchNo());
                warehouseEntity.setProduceDate(udiEntity.getProduceDate());
                warehouseEntity.setExpireDate(udiEntity.getExpireDate());
                warehouseEntity.setSerialNo(udiEntity.getSerialNo());
                warehouseEntity.setCount(1);
                List<WarehouseEntity> warehouseEntityList = new ArrayList<>();
                warehouseEntityList.add(warehouseEntity);
                codesTempService.insertCodesTempSingle(warehouseEntity);
            }

            return ResultVOUtils.success(orderId);
        } catch (Exception e) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

    }

    //网页端数据上传调用接口
    @PostMapping("warehouse/inout/postOrdersWeb")
    public BaseResponse postOrdersWeb(@RequestBody PostOrderRequest postOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            ioTransInoutService.creatOrderWeb(postOrderRequest);
        } catch (Exception e) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("导入成功");
    }

    //手持端数据上传调用接口
    @PostMapping("warehouse/inout/postOrders")
    public BaseResponse postOrders(@RequestBody PostOrderRequest postOrderRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            ioTransInoutService.creatOrder(postOrderRequest);
        } catch (Exception e) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("导入成功");
    }


    public void updateErpOrderId(List<ErpOrderEntity> erpOrderEntities, String oldId, String newId) {
        if (erpOrderEntities != null) {
            List<ErpOrderEntity> inserts = new ArrayList<>();
            for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                if (erpOrderEntity.getOrderIdFk().equals(oldId)) {
                    erpOrderEntity.setOrderIdFk(newId);
                    inserts.add(erpOrderEntity);
                }
            }
            myErpOrderService.insertErpOrders(inserts);
        }

    }


    @PostMapping("/admin/warehouse/inout/save")
    public BaseResponse save(@RequestBody @Valid WarehouseSaveRequest warehouseSaveRequest,
                             BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        WarehouseEntity warehouseEntity = new WarehouseEntity();
        BeanUtils.copyProperties(warehouseSaveRequest, warehouseEntity);

        boolean b = codesService.insertWarehouse(warehouseEntity);
        if (!b) {
            return ResultVOUtils.error(ResultEnum.NOT_NETWORK);
        }

        return ResultVOUtils.success("添加成功");
    }

}
