package com.glxp.sale.admin.controller.inout.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.sale.admin.constant.IncodeStatus;
import com.glxp.sale.admin.entity.basic.StockPrintEntity;
import com.glxp.sale.admin.entity.inventory.InCodeLogEntity;
import com.glxp.sale.admin.req.basic.FilterStockprintRequest;
import com.glxp.sale.admin.req.inventory.PostStockPrintRequest;
import com.glxp.sale.admin.res.PageSimpleResponse;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.inventory.InCodeLogService;
import com.glxp.sale.admin.service.inventory.InvStockPrintService;
import com.glxp.sale.admin.util.CustomUtil;
import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.sql.Date;
import java.util.List;

@RestController
public class InvPrintController {
    @Resource
    InvStockPrintService stockPrintService;
    @Resource
    private SystemParamConfigService systemParamConfigService;
    @Resource
    InCodeLogService inCodeLogService;

    @PostMapping("/udiwarehouse/stockPrint/filter")
    public BaseResponse filterAll(@RequestBody FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String thirdSys = systemParamConfigService.selectValueByParamKey("incode_thirdSys_fromType");
        stockprintRequest.setThirdSysFk(thirdSys);
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @GetMapping("/spms/stockPrint/filter")
    public BaseResponse filterSp( FilterStockprintRequest stockprintRequest,
                                  BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
//        String thirdSys = systemParamConfigService.selectValueByParamKey("incode_thirdSys_fromType");
//        stockprintRequest.setThirdSysFk(thirdSys);
        List<StockPrintEntity> stockPrintEntities
                = stockPrintService.filterStockPrint(stockprintRequest);
        PageInfo<StockPrintEntity> pageInfo;
        pageInfo = new PageInfo<>(stockPrintEntities);
        PageSimpleResponse<StockPrintEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(stockPrintEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("udiwms/stockprint/post")
    public BaseResponse postInCode(@RequestBody PostStockPrintRequest postStockPrintRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String genKey = null;
        if (postStockPrintRequest != null) {

            InCodeLogEntity inCodeLogEntity = new InCodeLogEntity();
            inCodeLogEntity.setFromType(IncodeStatus.FROM_THRSYS);
            inCodeLogEntity.setUpdateTime(new Date(System.currentTimeMillis()));
            if (postStockPrintRequest.getGenKey() == null) {
                genKey = CustomUtil.getId();
                inCodeLogEntity.setGenKey(genKey);
            } else {
                genKey = postStockPrintRequest.getGenKey();
                inCodeLogEntity.setGenKey(postStockPrintRequest.getGenKey());
            }
            inCodeLogEntity.setStatus(IncodeStatus.PROCESS);
            inCodeLogEntity.setThirdSysFk(postStockPrintRequest.getThirdSys());
            inCodeLogService.insertInCodeLog(inCodeLogEntity);

            List<StockPrintEntity> stockPrintEntities = postStockPrintRequest.getDatas();
            if (stockPrintEntities != null && stockPrintEntities.size() > 0) {
                for (StockPrintEntity stockPrintEntity : stockPrintEntities) {
                    stockPrintEntity.setGenKeyFk(genKey);
                    String udiCode = "#" + stockPrintEntity.getNameCode() + "#" + stockPrintEntity.getProduceDate() + "#" + stockPrintEntity.getExpireDate() +
                            "#" + stockPrintEntity.getBatchNo() + "#" + stockPrintEntity.getSerialNo() + "#" + stockPrintEntity.getWarehouseCode() + "#" + stockPrintEntity.getSpaceCode();
                    stockPrintEntity.setUdiCode(udiCode);
                    stockPrintService.insertStockPrint(stockPrintEntity);
                }
            }
            inCodeLogEntity.setStatus(IncodeStatus.FINISH);
            inCodeLogService.updateInCodeLog(inCodeLogEntity);
        }
        return ResultVOUtils.success(genKey);
    }


}
