package com.glxp.sale.admin.controller.inout.utils;

import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.entity.basic.BasicInstrumentMaintainRelevanceEntity;
import com.glxp.sale.admin.entity.basic.BussinessTypeEntity;
import com.glxp.sale.admin.entity.basic.UdiInfoEntity;
import com.glxp.sale.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.entity.inout.ErpOrderEntity;
import com.glxp.sale.admin.entity.inout.FilterErpOrderRequest;
import com.glxp.sale.admin.entity.inout.OrderEntity;
import com.glxp.sale.admin.entity.inout.WarehouseEntity;
import com.glxp.sale.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.sale.admin.req.inout.OrderFilterRequest;
import com.glxp.sale.admin.req.param.SystemParamConfigRequest;
import com.glxp.sale.admin.res.basic.UdiRelevanceResponse;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.inout.OrderDetailService;
import com.glxp.sale.admin.service.inout.OrderService;
import com.glxp.sale.admin.thread.IoTransInoutService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ContrastErpUtil {

    int erpTotalCount;
    List<ErpOrderEntity> mErpOrderEntities;
    String udiUrl = "";

    @Resource
    IoTransInoutService ioTransInoutService;

    public ContrastErpUtil() {

    }

    public ContrastErpUtil(String udiUrl) {
        this.udiUrl = udiUrl;
    }

    public void contrastErp(UdiInfoService udiInfoService, UdiRelevanceService udiRelevanceService,
                            OrderDetailService orderDetailService,
                            OrderService orderService,
                            List<WarehouseEntity> warehouseEntityList, SystemParamConfigService setupService, BussinessTypeEntity bussinessTypeEntity) {
        String orderIdFk = warehouseEntityList.get(0).getOrderId();
        FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
        filterErpOrderRequest.setOrderId(orderIdFk);
        mErpOrderEntities = orderDetailService.filterAllMyErpOrder(filterErpOrderRequest);
        if (mErpOrderEntities != null && mErpOrderEntities.size() > 0)
            for (ErpOrderEntity erpOrderEntity : mErpOrderEntities) {
                erpTotalCount = erpTotalCount + erpOrderEntity.getErpCount();
                erpOrderEntity.setCount(0);
            }
        String errMsg = null;
        String resultMsg = null;
        List<WarehouseEntity> errorData = new ArrayList<>();
        for (WarehouseEntity warehouseEntity : warehouseEntityList) {
            UdiInfoEntity udiInfoEntity = new InoutUtils(udiUrl).getUdiInfo(udiInfoService, udiRelevanceService, warehouseEntity.getCode());
            if (warehouseEntity.getCount() > 1) {
                udiInfoEntity.setCount(warehouseEntity.getCount());
            }
            List<UdiRelevanceEntity> udiRelevanceEntities = udiRelevanceService.selectByUuid(udiInfoEntity.getUuid());
            if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 0) {
                UdiRelevanceEntity udiRelevanceEntity = checkGoodsId(udiRelevanceEntities, bussinessTypeEntity);
                if (udiRelevanceEntity != null) {
                    udiInfoEntity.setThirdId(udiRelevanceEntity.getThirdId());
                    udiInfoEntity.setThirdName(udiRelevanceEntity.getThirdName());
                    udiInfoEntity.setWarehouseCode(warehouseEntity.getWarehouseCode());
                    errMsg = vaildErpOrder(warehouseEntityList, udiInfoEntity);
                    if (errMsg != null && (errMsg.equals("产品信息不匹配") || errMsg.equals("非此单产品批次号"))) {
                        errorData.add(warehouseEntity);
                    }
                } else {
                    errMsg = "产品信息不匹配";
                    errorData.add(warehouseEntity);
                }
            } else {
                errMsg = "产品信息未维护";
                errorData.add(warehouseEntity);
            }
            if (resultMsg == null && errMsg != null) {
                resultMsg = errMsg;
            }

        }
        orderDetailService.insertErpOrders(mErpOrderEntities);
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setId(orderIdFk);
        OrderEntity orderEntity = orderService.findOne(orderFilterRequest);
        ErpOrderEntity vailExitMsg = vailErpExit(orderDetailService, mErpOrderEntities, orderIdFk);
        SystemParamConfigRequest systemParamConfigRequest = new SystemParamConfigRequest();
        systemParamConfigRequest.setParamKey("erp_forbidSameErp");
        SystemParamConfigEntity systemParamConfigEntity = setupService.selectByParamKey(systemParamConfigRequest);
        if (vailExitMsg != null && systemParamConfigEntity != null && systemParamConfigEntity.getParamValue().equals("1")) {
            errMsg = "所选ERP单" + vailExitMsg.getErpOrderId() + "已被其他出入库单" + vailExitMsg.getOrderIdFk() + "绑定！";
        }
        if (mErpOrderEntities != null && mErpOrderEntities.size() > 0)
            for (ErpOrderEntity erpOrderEntity : mErpOrderEntities) {
                if (erpOrderEntity.getErpCount() != erpOrderEntity.getCount()) {
                    errMsg = "所选ERP单" + erpOrderEntity.getErpOrderId() + "下" + erpOrderEntity.getGoodsname() + "数量不匹配";
                }
            }

        if (errMsg != null) {
            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
            orderEntity.setRemark(errMsg);
            orderService.updateOrder(orderEntity);
        } else {
            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_SUCCESS);
            orderService.updateOrder(orderEntity);
            ioTransInoutService.printOrder(orderEntity.getId());
        }
        if (errorData.size() > 0) {
            new InoutUtils(udiUrl).generateConstrastDetail(udiInfoService, udiRelevanceService, orderDetailService, errorData);
        }
    }

    public UdiRelevanceEntity checkGoodsId(List<UdiRelevanceEntity> udiRelevanceEntities, BussinessTypeEntity bussinessTypeEntity) {
        if (mErpOrderEntities.size() > 0) {
            for (ErpOrderEntity erpOrderEntity : mErpOrderEntities) {
                for (UdiRelevanceEntity udiRelevanceEntity : udiRelevanceEntities) {
                    if (bussinessTypeEntity.getThirdSysFk() == null) {
                        if (erpOrderEntity.getGoodsid().equals(udiRelevanceEntity.getThirdId())) {
                            return udiRelevanceEntity;
                        }
                    } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId")) {
                        if (erpOrderEntity.getGoodsid().equals(udiRelevanceEntity.getThirdId())) {
                            return udiRelevanceEntity;
                        }
                    } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId1")) {
                        if (erpOrderEntity.getGoodsid().equals(udiRelevanceEntity.getThirdId1())) {
                            udiRelevanceEntity.setThirdId(udiRelevanceEntity.getThirdId1());
                            udiRelevanceEntity.setThirdName(udiRelevanceEntity.getThirdName1());
                            return udiRelevanceEntity;
                        }
                    } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId2")) {
                        if (erpOrderEntity.getGoodsid().equals(udiRelevanceEntity.getThirdId2())) {
                            udiRelevanceEntity.setThirdId(udiRelevanceEntity.getThirdId2());
                            udiRelevanceEntity.setThirdName(udiRelevanceEntity.getThirdName2());
                            return udiRelevanceEntity;
                        }
                    } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId3")) {
                        if (erpOrderEntity.getGoodsid().equals(udiRelevanceEntity.getThirdId3())) {
                            udiRelevanceEntity.setThirdId(udiRelevanceEntity.getThirdId3());
                            udiRelevanceEntity.setThirdName(udiRelevanceEntity.getThirdName3());
                            return udiRelevanceEntity;
                        }
                    } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId4")) {
                        if (erpOrderEntity.getGoodsid().equals(udiRelevanceEntity.getThirdId4())) {
                            udiRelevanceEntity.setThirdId(udiRelevanceEntity.getThirdId4());
                            udiRelevanceEntity.setThirdName(udiRelevanceEntity.getThirdName4());
                            return udiRelevanceEntity;
                        }
                    }

                }
            }
        }
        return null;
    }

    public int getSelectedCount(List<WarehouseEntity> warehouseEntityList) {
        int total = 0;
        for (WarehouseEntity warehouseEntity : warehouseEntityList) {
            total = total + warehouseEntity.getCount();
        }
        return total;
    }

    //验证产品ID,批次号，仓位号
    public String vaildErpOrder(List<WarehouseEntity> warehouseEntityList, UdiInfoEntity udiInfoEntity) {
        String errMsg = "产品信息不匹配";
        if (mErpOrderEntities.size() > 0) {
            for (ErpOrderEntity erpOrderEntity : mErpOrderEntities) {
                errMsg = isPass(erpOrderEntity, udiInfoEntity);
                if (errMsg == null) {
                    int erpCount = erpOrderEntity.getCount() + udiInfoEntity.getCount();
                    if (erpOrderEntity.getErpCount() != erpCount) {
                        erpOrderEntity.setCount(erpCount);
                        if (erpOrderEntity.getErpCount() != erpCount)
                            return "该" + udiInfoEntity.getBatchNo() + "批次号下" + udiInfoEntity.getCpmctymc() + "订单数目不匹配";
                    }
                    erpOrderEntity.setCount(erpCount);
                    return null;
                }
            }
        }
        if (errMsg == null && getSelectedCount(warehouseEntityList) + udiInfoEntity.getCount() != erpTotalCount) {
            return "订单数目匹配失败";
        }
        return errMsg;
    }

    public String isPass(ErpOrderEntity erpOrderEntity, UdiInfoEntity udiInfoEntity) {
        String errMsg = null;
        if (erpOrderEntity.getBatchNo().equals(udiInfoEntity.getBatchNo())) {
            if (erpOrderEntity.getGoodsid().equals(udiInfoEntity.getThirdId())) {
                if (erpOrderEntity.getWarehouseCode() != null && udiInfoEntity.getWarehouseCode() != null) {
                    if (erpOrderEntity.getWarehouseCode().equals(udiInfoEntity.getWarehouseCode())) {
                        return null;
                    } else
                        errMsg = "仓位号不匹配";
                } else
                    return null;
            } else
                errMsg = "产品信息不匹配";
        } else {
            errMsg = "非此单产品批次号";
        }
        return errMsg;
    }

    public List<ErpOrderEntity> transErp(List<ErpOrderEntity> erpOrderEntities, UdiRelevanceService udiRelevanceService, String orderFk) {
        List<ErpOrderEntity> returnOrders = new ArrayList<>();
        Map<String, ErpOrderEntity> filterMap = new HashMap<>();
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {

                FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                filterUdiInfoRequest.setThirdId(erpOrderEntity.getGoodsid());
                filterUdiInfoRequest.setPage(1);
                filterUdiInfoRequest.setLimit(100);
                List<UdiRelevanceResponse> udiRelevanceEntities = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);

                if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 0) {
                    UdiRelevanceResponse temp = udiRelevanceEntities.get(0);
                    erpOrderEntity.setCoName(temp.getCpmctymc());
                    erpOrderEntity.setAuthCode(temp.getZczbhhzbapzbh());
                    erpOrderEntity.setPackSpec(temp.getGgxh());
                    erpOrderEntity.setPackRatio(temp.getPackRatio());
                    erpOrderEntity.setProductCompany(temp.getYlqxzcrbarmc());
                    erpOrderEntity.setOrderIdFk(orderFk);
                }
                erpOrderEntity.setOrderIdFk(orderFk);
                String key = erpOrderEntity.getBatchNo() + erpOrderEntity.getGoodsid() + erpOrderEntity.getWarehouseCode();
                ErpOrderEntity filterTemp = filterMap.get(key);
                if (filterTemp != null) {
                    filterTemp.setErpCount(filterTemp.getErpCount() + erpOrderEntity.getErpCount());
                    filterTemp.setErpOrderId(filterTemp.getErpOrderId() + "," + erpOrderEntity.getErpOrderId());
                } else {
                    filterTemp = erpOrderEntity;
                }
                filterMap.put(key, filterTemp);
            }
            for (Map.Entry<String, ErpOrderEntity> entry : filterMap.entrySet()) {
                returnOrders.add(entry.getValue());
            }
        }


        return returnOrders;
    }


    public List<String> getDocids(List<ErpOrderEntity> erpOrderEntities) {
        Map<String, String> filter = new HashMap<>();
        List<String> docids = new ArrayList<>();
        for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {

            if (erpOrderEntity.getErpOrderId().contains(",")) {
                String[] dociArray = erpOrderEntity.getErpOrderId().split(",");
                if (dociArray != null && dociArray.length > 0) {
                    for (int i = 0; i < dociArray.length; i++) {
                        filter.put(dociArray[i], dociArray[i]);
                    }
                }
            } else {
                filter.put(erpOrderEntity.getErpOrderId(), erpOrderEntity.getErpOrderId());
            }
        }
        for (String docid : filter.keySet()) {
            docids.add(docid);
        }
        return docids;
    }

    //校验Erp单是否存在
    public ErpOrderEntity vailErpExit(OrderDetailService orderDetailService, List<ErpOrderEntity> erpOrderEntities, String orderId) {
        if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setOrderId(orderId);
            List<ErpOrderEntity> docids = orderDetailService.filterAllDocidE(filterErpOrderRequest);
            List<String> bDocids = getDocids(erpOrderEntities);
            if (docids != null && docids.size() > 0) {
                for (ErpOrderEntity docid : docids) {
                    for (String temp : bDocids) {
                        if (docid.getErpOrderId() != null && docid.getErpOrderId().contains(temp)) {
                            return docid;
                        }
                    }
                }
            }
            return null;
        }
        return null;
    }
}
