package com.glxp.sale.admin.thread;

import com.glxp.sale.admin.entity.basic.ErpStockEntity;
import com.glxp.sale.admin.entity.basic.UdiInfoEntity;
import com.glxp.sale.admin.entity.basic.BasicInstrumentMaintainRelevanceEntity;
import com.glxp.sale.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.sale.admin.entity.info.SetupEntity;
import com.glxp.sale.admin.entity.param.SystemParamConfigEntity;
import com.glxp.sale.admin.entity.inout.ErpOrderEntity;
import com.glxp.sale.admin.entity.inout.WarehouseEntity;
import com.glxp.sale.admin.httpclient.ErpOrderClient;
import com.glxp.sale.admin.req.inout.FilterOrderRequest;
import com.glxp.sale.admin.req.inout.WarehouseQueryRequest;
import com.glxp.sale.admin.req.param.SystemParamConfigRequest;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.info.SetupService;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.inout.CodesService;
import com.glxp.sale.admin.service.inventory.InvErpStockService;
import com.glxp.sale.admin.util.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class InvDlErpStockService {
    @Resource
    SetupService setupService;
    @Resource
    InvErpStockService erpStockService;
    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    UdiInfoService udiInfoService;
    @Resource
    CodesService warehouseService;
    @Resource
    SystemParamConfigService systemParamConfigService;
    final Logger logger = LoggerFactory.getLogger(InvDlErpStockService.class);

    /**
     * 下载ERP采购入库单所有产品信息到ERP库存表
     */
    @Async
    public void dlErpStock() {
//        SetupEntity setupEntity = setupService.selectSetup();
//        Integer day = setupEntity.getDlStockErpDay();

        SystemParamConfigRequest systemParamConfigRequest = new SystemParamConfigRequest();
        systemParamConfigRequest.setParamKey("dlStockErp");
        SystemParamConfigEntity systemParamConfigEntityDay = systemParamConfigService.selectByParamKey(systemParamConfigRequest);

        Integer day = Integer.valueOf(systemParamConfigEntityDay.getParamValue());
        Map date = DateUtil.getDayRange(day);
        Integer page = 1;
        Integer limit = 20;

        while (true) {

            systemParamConfigRequest.setParamKey("erp_order_query");
            SystemParamConfigEntity systemParamConfigEntity = systemParamConfigService.selectByParamKey(systemParamConfigRequest);
            if (systemParamConfigEntity == null) {
                logger.error("ERP出入库单据接口未定义");
                return;
            }
            FilterOrderRequest filterOrderRequest = new FilterOrderRequest();
            filterOrderRequest.setPage(page);
            filterOrderRequest.setLimit(limit);
            filterOrderRequest.setStartDate((String)date.get("startDate"));
            filterOrderRequest.setEndDate((String)date.get("endDate"));
            filterOrderRequest.setBillAction("PurchaseWareHouseIn");
            List<ErpOrderEntity> erpOrderEntities = new ErpOrderClient(systemParamConfigEntity.getParamValue()).getErpOrder(filterOrderRequest);
            List<ErpStockEntity> erpStockEntities = new ArrayList<>();
            if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                page++;
                for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                    //ERP数据更新
                    ErpStockEntity erpStockEntity = new ErpStockEntity();
                    erpStockEntity.setOrderId(erpOrderEntity.getErpOrderId());
                    erpStockEntity.setCode(erpOrderEntity.getGoodsid());
                    erpStockEntity.setBatchNo(erpOrderEntity.getBatchNo());
                    erpStockEntity.setExpirationDate(erpOrderEntity.getExpireDate());
                    erpStockEntity.setName(erpOrderEntity.getGoodsname());
                    erpStockEntity.setCount(erpOrderEntity.getCount() + "");

                    //本地数据更新
                    UdiRelevanceEntity udiRelevanceEntity =
                            udiRelevanceService.selectByThirdId(erpStockEntity.getCode(), "thirdId");
                    if (udiRelevanceEntity != null) {
                        UdiInfoEntity udiInfoEntity = udiInfoService.findByUuid(udiRelevanceEntity.getUuid());
                        WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
                        warehouseQueryRequest.setUdi(udiInfoEntity.getNameCode());
                        warehouseQueryRequest.setBatchNo(erpStockEntity.getBatchNo());
                        List<WarehouseEntity> warehouseEntities = warehouseService.findByOrderId(warehouseQueryRequest);
                        if (warehouseEntities != null && warehouseEntities.size() > 0) {
                            erpStockEntity.setManufacturingDate(warehouseEntities.get(0).getProduceDate());
                            erpStockEntity.setExpirationDate(warehouseEntities.get(0).getExpireDate());
                        }
                    }
                    erpStockEntities.add(erpStockEntity);
                    erpStockService.insertErpStocks(erpStockEntities);
                }
            } else {
                break;
            }
        }

    }


}
