package com.glxp.sale.admin.thread;

import com.glxp.sale.admin.config.WebSocketServer;
import com.glxp.sale.admin.constant.ConstantStatus;
import com.glxp.sale.admin.controller.inout.utils.ContrastErpUtil;
import com.glxp.sale.admin.controller.inout.utils.DataTransUtil;
import com.glxp.sale.admin.controller.inout.utils.InoutUtils;
import com.glxp.sale.admin.entity.basic.BasicThirdSysDetailEntity;
import com.glxp.sale.admin.entity.basic.BussinessTypeEntity;
import com.glxp.sale.admin.entity.basic.UdiRelevanceEntity;
import com.glxp.sale.admin.entity.inout.*;
import com.glxp.sale.admin.entity.udid.UdiEntity;
import com.glxp.sale.admin.httpclient.ErpOrderClient;
import com.glxp.sale.admin.req.basic.FilterUdiInfoRequest;
import com.glxp.sale.admin.req.inout.*;
import com.glxp.sale.admin.res.basic.UdiRelevanceResponse;
import com.glxp.sale.admin.service.basic.BussinessTypeService;
import com.glxp.sale.admin.service.basic.UdiInfoService;
import com.glxp.sale.admin.service.basic.UdiRelevanceService;
import com.glxp.sale.admin.service.inout.*;
import com.glxp.sale.admin.service.param.SystemParamConfigService;
import com.glxp.sale.admin.service.thrsys.ThrDataService;
import com.glxp.sale.admin.service.thrsys.ThrOrderService;
import com.glxp.sale.admin.util.FilterUdiUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class IoTransInoutService {

    private static final Logger logger = LoggerFactory.getLogger(IoTransInoutService.class);
    @Resource
    private CodesService warehouseService;
    @Resource
    private OrderService orderService;
    @Resource
    private OrderDetailService myErpOrderService;


    @Resource
    UdiRelevanceService udiRelevanceService;
    @Resource
    private UdiInfoService udiInfoService;
    @Value("${UDI_SERVER_URL}")
    private String udiUrl;

    @Resource
    private CodesTempService codesTempService;
    @Autowired
    private SystemParamConfigService systemParamConfigService;
    @Resource
    private BussinessTypeService bussinessTypeService;
    @Resource
    private ThrOrderService thrOrderService;
    @Resource
    ThrDataService thrDataService;
    @Resource
    StockOrderService stockOrderService;
    @Resource
    StockOrderDetailService stockOrderDetailService;


    //web端新增订单，
    @Async
    public void creatOrderWeb(PostOrderRequest postOrderRequest) {
        List<PostOrderRequest.PostOrder> postOrders = postOrderRequest.getPostOrder();
        if (postOrders != null && postOrders.size() > 0) {
            for (int i = 0; i < postOrders.size(); i++) {
                PostOrderRequest.PostOrder postOrder = postOrders.get(i);
                List<WarehouseEntity> warehouseEntityList = postOrder.getCodes();
                if (warehouseEntityList != null && warehouseEntityList.size() > 0) {
                    if (StringUtils.isNotBlank(postOrderRequest.getOldOrderId())) {
                        orderService.deleteById(postOrderRequest.getOldOrderId());
                        myErpOrderService.deleteByOrderId(postOrderRequest.getOldOrderId());
                    }
                    WarehouseEntity warehouseEntity = warehouseEntityList.get(0);
                    String orderId = System.currentTimeMillis() + i + "";
                    OrderEntity orderSaveRequest = new OrderEntity();
                    orderSaveRequest.setId(orderId);
                    if (warehouseEntity.getActDate() != null) {
                        orderSaveRequest.setActDate(warehouseEntity.getActDate());
                    }
                    orderSaveRequest.setAction(warehouseEntity.getAction());
                    orderSaveRequest.setActor(warehouseEntity.getActor());
                    orderSaveRequest.setCorpOrderId(warehouseEntity.getCorpOrderId());
                    orderSaveRequest.setFromCorpId(warehouseEntity.getFromCorpId());
                    orderSaveRequest.setMainAction(warehouseEntity.getMainAction());
                    orderSaveRequest.setFromCorp(warehouseEntity.getFromCorp());
                    orderSaveRequest.setFromType("网页端");
                    orderSaveRequest.setStatus(postOrderRequest.getSubmitType() == 0 ?
                            ConstantStatus.ORDER_STATUS_PROCESS : ConstantStatus.ORDER_STATUS_TEMP_SAVE);
                    orderSaveRequest.setExportStatus(ConstantStatus.ORDER_EXPORT_UN);
                    orderSaveRequest.setStockCheckFk(postOrder.getStockIdFk());
                    orderSaveRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                    orderSaveRequest.setSignStatus(ConstantStatus.ORDER_SIGN_UN);
                    List<String> docidsList = postOrder.getErpOrders();
                    String docidStr = "";
                    if (docidsList != null) {
                        for (String key : docidsList) {
                            docidStr = docidStr + "," + key;
                        }
                        if (docidStr.length() > 1) {
                            docidStr = docidStr.substring(1);
                            orderSaveRequest.setErpFk(docidStr);
                        }
                    }
                    orderService.insertOrder(orderSaveRequest);
                    for (int index = 0; index < warehouseEntityList.size(); index++) {
                        warehouseEntityList.get(index).setOrderId(orderId);
                    }
                    codesTempService.insertCodesTemp(warehouseEntityList);
                }
            }
        }
    }

    //生成订单，插入零时表
    @Async
    public void creatOrder(PostOrderRequest postOrderRequest) {
        List<PostOrderRequest.PostOrder> postOrders = postOrderRequest.getPostOrder();
        if (postOrders != null && postOrders.size() > 0) {
            for (int i = 0; i < postOrders.size(); i++) {
                PostOrderRequest.PostOrder postOrder = postOrders.get(i);
                List<WarehouseEntity> warehouseEntityList = postOrder.getCodes();
                if (warehouseEntityList != null && warehouseEntityList.size() > 0) {

                    //解析条码
                    for (WarehouseEntity temp : warehouseEntityList) {
                        UdiEntity udiEntity = FilterUdiUtils.getUdi(temp.getCode());
                        temp.setNameCode(udiEntity.getUdi());
                    }


                    WarehouseEntity warehouseEntity = warehouseEntityList.get(0);
                    String orderId = System.currentTimeMillis() + i + "";
                    OrderEntity orderSaveRequest = new OrderEntity();
                    orderSaveRequest.setId(orderId);
                    if (warehouseEntity.getActDate() != null) {
                        orderSaveRequest.setActDate(warehouseEntity.getActDate());
                    }
                    orderSaveRequest.setAction(warehouseEntity.getAction());
                    orderSaveRequest.setActor(warehouseEntity.getActor());
                    orderSaveRequest.setCorpOrderId(warehouseEntity.getCorpOrderId());
                    orderSaveRequest.setFromCorpId(warehouseEntity.getFromCorpId());
                    orderSaveRequest.setMainAction(warehouseEntity.getMainAction());
                    orderSaveRequest.setFromCorp(warehouseEntity.getFromCorp());
                    orderSaveRequest.setFromType("手持终端");
                    orderSaveRequest.setStatus(ConstantStatus.ORDER_STATUS_PROCESS);
                    orderSaveRequest.setExportStatus(ConstantStatus.ORDER_EXPORT_UN);
                    orderSaveRequest.setStockCheckFk(postOrder.getStockIdFk());
                    orderSaveRequest.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                    orderSaveRequest.setSignStatus(ConstantStatus.ORDER_SIGN_UN);
                    List<String> docidsList = postOrder.getErpOrders();
                    String docidStr = "";
                    if (docidsList != null) {
                        for (String key : docidsList) {
                            docidStr = docidStr + "," + key;
                        }
                        if (docidStr.length() > 1) {
                            docidStr = docidStr.substring(1);
                            orderSaveRequest.setErpFk(docidStr);
                        }
                    }
                    orderService.insertOrder(orderSaveRequest);
                    for (int index = 0; index < warehouseEntityList.size(); index++) {
                        warehouseEntityList.get(index).setOrderId(orderId);
                    }
                    codesTempService.insertCodesTemp(warehouseEntityList);
                }
            }
        }
    }

    //查找等待处理单据，转入正式表
    @Async
    public void transInout() {
        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setStatus(ConstantStatus.ORDER_STATUS_PROCESS);
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        for (int i = 0; i < orderEntities.size(); i++) {
            OrderEntity orderEntity = orderEntities.get(i);
            orderEntity = orderService.findById(orderEntity.getId());
            if (orderEntity.getStatus() == ConstantStatus.ORDER_STATUS_PROCESS) {
                transInout(orderEntity);
            }

        }
    }

    public void transInout(OrderEntity orderEntity) {

        WarehouseQueryRequest warehouseQueryRequest = new WarehouseQueryRequest();
        warehouseQueryRequest.setOrderId(orderEntity.getId());
        List<WarehouseEntity> filterList = codesTempService.findByOrderId(warehouseQueryRequest);
        List<WarehouseEntity> insertList = new ArrayList<>();

        for (int j = 0; j < filterList.size(); j++) {
            WarehouseEntity warehouseEntity = filterList.get(j);
            String result = new InoutUtils().filterCode(warehouseService, warehouseEntity, systemParamConfigService);
            if (result == null) {
                insertList.add(warehouseEntity);
            } else {
                orderService.updateOrderStatus(warehouseEntity.getOrderId(), ConstantStatus.ORDER_STATUS_FAIL);
                orderService.updateRemark(orderEntity.getId(), result);
                insertList.clear();
                return;
            }
        }

        if (insertList.size() > 0) {
            warehouseService.insertWarehouses(insertList);
            WarehouseQueryRequest tempQuery = new WarehouseQueryRequest();
            tempQuery.setOrderId(orderEntity.getId());
            List<WarehouseEntity> temps = codesTempService.findByOrderId(tempQuery);
            if (orderEntity.getErpFk() != null && !"".equals(orderEntity.getErpFk())) {
                orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_CHECK);
            } else {
                //因为只需用到条码，所以不需要重新读取
                new InoutUtils(udiUrl).generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, insertList);
                orderService.updateOrderStatus(orderEntity.getId(), ConstantStatus.ORDER_STATUS_SUCCESS);
                printOrder(orderEntity.getId());
            }

            List<Long> ids = new ArrayList<>();
            for (WarehouseEntity w : temps) {
                ids.add(w.getId());
            }
            codesTempService.deleteCodesTemp(ids);
            insertList.clear();
        }
    }

    //校验ERP单
    @Async
    public void checkOrder() {

        OrderFilterRequest orderFilterRequest = new OrderFilterRequest();
        orderFilterRequest.setStatus(ConstantStatus.ORDER_STATUS_CHECK);
        List<OrderEntity> orderEntities = orderService.findAllOrders(orderFilterRequest);
        for (int i = 0; i < orderEntities.size(); i++) {
            OrderEntity orderEntity = orderEntities.get(i);
            checkOrder(orderEntity);
        }
    }


    @Async
    public void repeatPrint(OrderEntity orderEntity) {
        printOrder(orderEntity.getId());
    }

    public void checkOrder(OrderEntity orderEntity) {
        orderEntity = orderService.findById(orderEntity.getId());
        List<String> erpIds = DataTransUtil.strToErpList(orderEntity.getErpFk());
        List<WarehouseEntity> transData = warehouseService.findByReceiptId(orderEntity.getId());
        if (erpIds != null && erpIds.size() > 0) {

            List<ErpOrderEntity> erpOrderEntities = thrDataService.getStockOrders(erpIds);
            checkThirdOrder(erpOrderEntities, orderEntity, transData);

        } else {
            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
            orderEntity.setRemark("未选入第三方出入库单据进行校验，请手动选择");
            orderService.updateOrder(orderEntity);
            myErpOrderService.deleteByOrderId(orderEntity.getId());
            new InoutUtils(udiUrl).generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, transData);
            printOrder(orderEntity.getId());
        }
    }

    public void checkThirdOrder(List<ErpOrderEntity> cloudErps, OrderEntity orderEntity, List<WarehouseEntity> transData) {
        if (cloudErps != null && cloudErps.size() > 0) {
            List<ErpOrderEntity> returnOrders = new ContrastErpUtil().transErp(cloudErps, udiRelevanceService, orderEntity.getId());
            ErpOrderEntity temp = returnOrders.get(0);
            for (int i = 0; i < returnOrders.size(); i++) {
                if (!temp.getCompanyname().equals(returnOrders.get(i).getCompanyname())) {
                    orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                    orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
                    orderEntity.setRemark("选择Erp订单往来单位不一致");
                    orderService.updateOrder(orderEntity);
                    return;
                }
            }
            myErpOrderService.deleteByOrderId(orderEntity.getId());
            myErpOrderService.insertErpOrders(returnOrders);
            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());
            new ContrastErpUtil(udiUrl).contrastErp(udiInfoService, udiRelevanceService, myErpOrderService,
                    orderService, transData, systemParamConfigService, bussinessTypeEntity);
        } else {
            orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
            orderEntity.setStatus(ConstantStatus.ORDER_STATUS_SUCCESS);
            orderEntity.setRemark("获取ERP出入库单据失败");
            orderService.updateOrder(orderEntity);
            myErpOrderService.deleteByOrderId(orderEntity.getId());
            new InoutUtils(udiUrl).generateConstrastDetail(udiInfoService, udiRelevanceService, myErpOrderService, transData);
        }
    }

    //重码校验错误重新提交
    @Async
    public void repeatCommit(CommitRequest commitRequest) {
        OrderEntity orderEntity = orderService.findById(commitRequest.getOrderId());
        if (orderEntity != null) {
            transInout(orderEntity);
            WebSocketServer.sendInfo("重码校验已完成，请刷新查看！", null);
        }
    }

    //重新校验
    @Async
    public void repeatCheck(String orderId) {
        OrderEntity orderEntity = orderService.findById(orderId);
        checkOrder(orderEntity);
        WebSocketServer.sendInfo("单据:" + orderId + "校验已完成，请刷新查看！", null);
    }

    //校验ERP单后，订单详情转入等待打印
    @Async
    public void printOrder(String orderId) {
        OrderEntity orderEntity = orderService.findById(orderId);
        if (orderEntity.getStatus() == ConstantStatus.ORDER_STATUS_SUCCESS && orderEntity.getErpFk() == null
        ) {

            //查询单据校验后生成的单据详情
            FilterErpOrderRequest filterErpOrderRequest = new FilterErpOrderRequest();
            filterErpOrderRequest.setOrderId(orderId);
            List<ErpOrderEntity> erpOrderEntities = myErpOrderService.filterAllMyErpOrder(filterErpOrderRequest);

            BussinessTypeEntity bussinessTypeEntity = bussinessTypeService.findBTByAction(orderEntity.getAction());


            //生成业务单据详情
            List<StockOrderDetailEntity> stockOrderDetailEntities = new ArrayList<>();
            if (erpOrderEntities != null && erpOrderEntities.size() > 0) {
                boolean isError = false;
                for (ErpOrderEntity erpOrderEntity : erpOrderEntities) {
                    StockOrderDetailEntity stockOrderDetailEntity = new StockOrderDetailEntity();
//                    stockOrderDetailEntity.setOrderIdFk(stockOrderEntity.getId());
                    stockOrderDetailEntity.setProductId(erpOrderEntity.getGoodsid());
                    stockOrderDetailEntity.setProductName(erpOrderEntity.getGoodsname());

                    if (erpOrderEntity.getGoodsid() == null)
                        stockOrderDetailEntity.setProductId(erpOrderEntity.getNameCode());
                    if (erpOrderEntity.getGoodsname() == null)
                        stockOrderDetailEntity.setProductName(erpOrderEntity.getCoName());
                    stockOrderDetailEntity.setSpec(erpOrderEntity.getPackSpec());
                    stockOrderDetailEntity.setBatchNo(erpOrderEntity.getBatchNo());
                    stockOrderDetailEntity.setProductDate(erpOrderEntity.getProductDate());
                    stockOrderDetailEntity.setExpireDate(erpOrderEntity.getExpireDate());
//                    stockOrderDetailEntity.setYlqxzcrbarmc(erpOrderEntity.get());
                    if (erpOrderEntity.getCount() != null) {
                        stockOrderDetailEntity.setSweepCount(erpOrderEntity.getCount() + "");
                        stockOrderDetailEntity.setReCount(erpOrderEntity.getReCount() + "");
                        stockOrderDetailEntity.setCount(erpOrderEntity.getErpCount() + "");
                    }
                    if (erpOrderEntity.getReCount() != null) {
                        stockOrderDetailEntity.setReCount(erpOrderEntity.getReCount() + "");
                    }
                    if (erpOrderEntity.getErpCount() != null) {
                        stockOrderDetailEntity.setCount(erpOrderEntity.getErpCount() + "");
                    }
                    FilterUdiInfoRequest filterUdiInfoRequest = new FilterUdiInfoRequest();
                    filterUdiInfoRequest.setNameCode(erpOrderEntity.getNameCode());
                    List<UdiRelevanceResponse> udiRelevanceEntities = udiRelevanceService.filterUdiRelevance(filterUdiInfoRequest);
                    if (udiRelevanceEntities != null && udiRelevanceEntities.size() > 1 && erpOrderEntity.getBindRlFk() == null) {
                        isError = true;
                        erpOrderEntity.setBindStatus(ConstantStatus.ORDER_DETAIL_DI_MUTI);
                        myErpOrderService.updateErpOrder(erpOrderEntity);
                    } else if (udiRelevanceEntities.size() == 1) {
                        UdiRelevanceResponse udiRelevanceEntity = udiRelevanceEntities.get(0);
                        setThridName(bussinessTypeEntity, stockOrderDetailEntity, udiRelevanceEntity);

                    } else if (erpOrderEntity.getBindRlFk() != null) {
                        UdiRelevanceEntity udiRelevanceEntity = udiRelevanceService.selectById(erpOrderEntity.getBindRlFk());
                        UdiRelevanceResponse udiRelevanceResponse = new UdiRelevanceResponse();
                        BeanUtils.copyProperties(udiRelevanceEntity, udiRelevanceResponse);
                        setThridName(bussinessTypeEntity, stockOrderDetailEntity, udiRelevanceResponse);
                        erpOrderEntity.setGoodsid(stockOrderDetailEntity.getProductId());
                        erpOrderEntity.setGoodsname(stockOrderDetailEntity.getProductName());
                        myErpOrderService.updateErpOrder(erpOrderEntity);
                    }

                    stockOrderDetailEntities.add(stockOrderDetailEntity);
                }
                if (!isError) {

                    StockOrderEntity stockOrderEntity = genStockOrder(orderEntity, bussinessTypeEntity);
                    for (StockOrderDetailEntity stockOrderDetailEntity : stockOrderDetailEntities) {
                        stockOrderDetailEntity.setOrderIdFk(stockOrderEntity.getId());
                    }
                    stockOrderDetailService.insertStockOrderDetails(stockOrderDetailEntities);
                    orderEntity.setRemark("");
                    orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_UN);
                    orderService.updateOrder(orderEntity);
                } else {
                    orderEntity.setRemark("该产品DI绑定多个产品ID，请在扫码单据详情绑定对应产品ID");
                    orderEntity.setContrastStatus(ConstantStatus.ORDER_CHECK_FAIL);
                    orderService.updateOrder(orderEntity);
                }
            }
        } else if (orderEntity.getContrastStatus() == ConstantStatus.ORDER_CHECK_SUCCESS) {
            String[] erpIds = orderEntity.getErpFk().split(",");
            if (erpIds != null && erpIds.length > 0) {
                for (int i = 0; i < erpIds.length; i++) {
                    StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
                    stockOrderFilterRequest.setBillNo(erpIds[i]);
                    StockOrderEntity stockOrderEntity = stockOrderService.findOne(stockOrderFilterRequest);
                    stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
                    stockOrderService.updateById(stockOrderEntity);
                }
            }

        }
    }

    public StockOrderEntity genStockOrder(OrderEntity orderEntity, BussinessTypeEntity bussinessTypeEntity) {
        //生成业务单据记录
        StockOrderEntity stockOrderEntity = new StockOrderEntity();
        stockOrderEntity.setBillNo(orderEntity.getId());
        stockOrderEntity.setBillType(bussinessTypeEntity.getLocalAction());
        stockOrderEntity.setBilldate(orderEntity.getActDate());
        stockOrderEntity.setCustomerId(orderEntity.getCustomerId());
        stockOrderEntity.setUnitIdFk(orderEntity.getCustomerId());
        stockOrderEntity.setSourceType(ConstantStatus.ST_SCAN_ADD);
        stockOrderEntity.setCorpId(orderEntity.getFromCorpId());
        stockOrderEntity.setStatus(ConstantStatus.SORDER_CHECKED);
        stockOrderEntity.setCorpName(orderEntity.getFromCorp());
        stockOrderService.insertStockOrder(stockOrderEntity);

        //删除关联的业务单据详情
        StockOrderFilterRequest stockOrderFilterRequest = new StockOrderFilterRequest();
        stockOrderFilterRequest.setBillNo(stockOrderEntity.getBillNo());
        stockOrderEntity = stockOrderService.findOne(stockOrderFilterRequest);
        stockOrderDetailService.deleteByOrderIdFk(stockOrderEntity.getId());

        return stockOrderEntity;
    }


    public void setThridName(BussinessTypeEntity bussinessTypeEntity, StockOrderDetailEntity stockOrderDetailEntity, UdiRelevanceResponse udiRelevanceEntity) {
        if (bussinessTypeEntity.getThirdSysFk().equals("thirdId")) {
            stockOrderDetailEntity.setProductId(udiRelevanceEntity.getThirdId());
            stockOrderDetailEntity.setProductName(udiRelevanceEntity.getThirdName());
        } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId1")) {
            stockOrderDetailEntity.setProductId(udiRelevanceEntity.getThirdId1());
            stockOrderDetailEntity.setProductName(udiRelevanceEntity.getThirdName1());
        } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId2")) {
            stockOrderDetailEntity.setProductId(udiRelevanceEntity.getThirdId2());
            stockOrderDetailEntity.setProductName(udiRelevanceEntity.getThirdName2());
        } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId3")) {
            stockOrderDetailEntity.setProductId(udiRelevanceEntity.getThirdId3());
            stockOrderDetailEntity.setProductName(udiRelevanceEntity.getThirdName3());
        } else if (bussinessTypeEntity.getThirdSysFk().equals("thirdId4")) {
            stockOrderDetailEntity.setProductId(udiRelevanceEntity.getThirdId4());
            stockOrderDetailEntity.setProductName(udiRelevanceEntity.getThirdName4());
        }
    }
}


