package com.glxp.sale.admin.upload;

import com.glxp.sale.common.enums.ResultEnum;
import com.glxp.sale.common.res.BaseResponse;
import com.glxp.sale.common.util.ResultVOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 上传文件控制器
 */
@RestController
public class uploadController {
    /**
     * 上传PDF模板
     */
    @PostMapping("/udiwms/upload/pdf/template")
    public BaseResponse uploadPDFTemplate(@RequestParam("file") MultipartFile file) throws UnsupportedEncodingException {
        if (file.isEmpty()) {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传文件不能为空");
        }

        // 保存文件 ---------------------
        String fileName = file.getOriginalFilename();
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        // 文件类型判断
        if (StringUtils.isBlank(fileType) || !fileType.equals(".pdf")) {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传文件只能是 pdf 格式");
        }
        String newName = UUID.randomUUID()+fileType;//生成新文件名
        String savePath = "d:/1s/udiwms/pdf/template";

        File file1 = new File(savePath);
        if(!file1.exists()){// 判断目录是否存在
            file1.mkdirs();// 创建多层目录
        }
        file1 = new File(savePath + "/" + newName) ;
        try {
            file.transferTo(file1);
            Map<String, String> rMap = new HashMap<>();
            rMap.put("msg", "上传成功");
            rMap.put("path", savePath + "/" + newName);
            return ResultVOUtils.success(rMap);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传失败");
    }

    /**
     * 注册页面上传文件
     */
    @PostMapping("/udiwms/upload/register/file")
    public BaseResponse uploadRegisterFile(@RequestParam("file") MultipartFile file,
                                           @RequestParam("type") String type) throws UnsupportedEncodingException {
        if (file.isEmpty()) {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传文件不能为空");
        }
        if (StringUtils.isBlank(type)) {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传文件类型不能为空");
        }

        // 保存文件 ---------------------
        String fileName = file.getOriginalFilename();
        String fileType = fileName.substring(fileName.lastIndexOf("."));
        // 文件类型判断
        if (StringUtils.isBlank(fileType) || !fileType.equals(".jpg") && !fileType.equals(".png")) {
            return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传文件只能是 .jpg,.png 格式");
        }
        String newName = UUID.randomUUID()+fileType;//生成新文件名
        String savePath = "d:/1s/udiwms/register/file/" + type;

        File file1 = new File(savePath);
        if(!file1.exists()){// 判断目录是否存在
            file1.mkdirs();// 创建多层目录
        }
        file1 = new File(savePath + "/" + newName) ;
        try {
            file.transferTo(file1);
            Map<String, String> rMap = new HashMap<>();
            rMap.put("msg", "上传成功");
            rMap.put("name", newName);
            rMap.put("type", type);
            return ResultVOUtils.success(rMap);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return ResultVOUtils.error(ResultEnum.DATA_ERROR, "上传失败");
    }
}